-- -----------------------------------------------------------------
-- File: AllInOneBank.lua
--
-- Purpose: Functions for AIOB WoW Window.
--
-- Author: Robinson
--
-- Credits:
--   Ramble
--   Starven, for MyInventory
--   Kaitlin, for BankItems
--   Sarf, for the original concept of AllInOneInventory
-- -----------------------------------------------------------------
-----------------------
-- Saved Configuration
-----------------------
AIOBProfile = {}
AIOB_VERSION          =  "2.1.0";

local PlayerName      = nil; -- Logged in player name
local bankPlayer      = nil; -- viewing player pointer
local bankPlayerName  = nil; -- Viewing player name

------------------------
-- Saved Function Calls
------------------------
local BankFrame_Saved = nil;
local PurchaseSlot_Saved = nil;
-----------------------
-- Local Configuration
-----------------------
local AIOB_Loaded = nil;
local AtBank = false;

AIOB_MAX_ID              = 224; -- 8 * 28 slot bags, 28 Bankslots
AIOB_COLUMNS_MIN         =   8; -- 7 Bags, so it has to be at least 8 wide :)
AIOB_COLUMNS_MAX         =  18; -- Same as MI
AIOB_BASE_HEIGHT         = 153; -- Height of Borders + Title + Bottom
AIOB_ROW_HEIGHT          =  40; -- One Row
AIOB_BASE_WIDTH          =  12; -- Width of the borders
AIOB_COL_WIDTH           =  39; -- One Column
AIOB_FIRST_ITEM_OFFSET_X =   7; -- Leave room for the border
AIOB_FIRST_ITEM_OFFSET_Y = -28 - 39; -- Leave room for the title.
AIOB_ITEM_OFFSET_X       =  39; -- Each is 39 apart
AIOB_ITEM_OFFSET_Y       = -39; -- Each is 39 apart

-- AIOBDEBUG                = 0; using AIOIDEBUG now
-- Not Saved between Sessions
AIOBAllRealms            = 0;
-- Saved Between Sessions
AIOBReplaceBank          = 1;
AIOBColumns              = 10;
AIOBFreeze               = 0;
AIOBHighlightItems       = 1;
AIOBHighlightBags        = 1;
AIOBBagView              = 0;
AIOBGraphics             = 1;
AIOBBackground           = 1;
AIOBShowPlayers          = 1;
AIOBScale                = 1;
AIOBCount                = 0;
AIOBButtons              = 1;
AIOBCash                 = 1;

--InitializeProfile: Initializes a players profile {{{
--  If Player's profile is not found, it makes a new one from defaults
--  If Player's profile is found, it loads the values from AIOBProfile
function AIOB_InitializeProfile()
   if ( UnitName("player") ) then
      PlayerName = UnitName('player').."|"..AIOI_Trim(GetCVar("realmName"));
      AIOB_setPlayerName(PlayerName)
      AIOB_LoadSettings();

      AIOI_DEBUG("AIOB: Profile for "..PlayerName.." Initilized.");
      AIOBFrame_PopulateFrame();
      -- TODO: why is this here?
      -- AIOB_UpdateBagCost(AIOBProfile[PlayerName].Bags);
   end
end

function AIOB_LoadSettings()
   if ( AIOBProfile[PlayerName] == nil ) then
      AIOBProfile[PlayerName] = {};
      AIOI_Print("Creating new Profile for "..PlayerName);
   end

   AIOBReplaceBank    = AIOB_SavedOrDefault("ReplaceBank");
   AIOBColumns        = AIOB_SavedOrDefault("Columns");
   AIOBFreeze         = AIOB_SavedOrDefault("Freeze");
   AIOBHighlightItems = AIOB_SavedOrDefault("HighlightItems");
   AIOBHighlightBags  = AIOB_SavedOrDefault("HighlightBags");
   AIOBBagView        = AIOB_SavedOrDefault("BagView");
   AIOBGraphics       = AIOB_SavedOrDefault("Graphics");
   AIOBBackground     = AIOB_SavedOrDefault("Background");
   AIOBShowPlayers    = AIOB_SavedOrDefault("ShowPlayers");
   AIOBScale          = AIOB_SavedOrDefault("Scale");
   AIOBCount          = AIOB_SavedOrDefault("Count");
   AIOBButtons        = AIOB_SavedOrDefault("Buttons");
   AIOBCash           = AIOB_SavedOrDefault("Cash");


   AIOB_SetGraphics();
   AIOB_SetReplaceBank();
   AIOB_SetFreeze();
end
function AIOB_SavedOrDefault(varname)
   if PlayerName == nil or varname == nil then
      AIOI_DEBUG("ERR: nil value");
      return nil;
   end
   if AIOBProfile[PlayerName][varname] == nil then -- Setting not set
      AIOBProfile[PlayerName][varname] = getglobal("AIOB"..varname); -- Load Default
   end
   return AIOBProfile[PlayerName][varname];  -- Return Setting.
end
-- END  Initialization }}}
function AIOB_GetPlayer(playerName)
   if ( not AIOBProfile[playerName] ) then
      AIOB_InitializeProfile();
   end
   bankPlayerName = playerName;
   AIOB_setBankPlayer(bankPlayerName)
   UIDropDownMenu_SetSelectedValue(AIOBDropDown, bankPlayerName);
   return AIOBProfile[playerName];
end

function AIOB_GetBag(bagIndex)
   if bagIndex == BANK_CONTAINER then
      return bankPlayer["Bank"];
   end
   return bankPlayer["Bag"..bagIndex];
end

function AIOB_GetBagsTotalSlots()
   local slots = 28;
   if bankPlayer == nil then
      return slots;
   end
   for bag = 5, 11 do
      local curBag = bankPlayer["Bag"..bag];
      if (curBag ~= nil) then
         if AIOI_GetItemSlots(curBag) ~=nil then
            slots = slots + AIOI_GetItemSlots(curBag);
         end
      end
   end
   return slots;
end

-- == Event Handling ==
-- OnLoad {{{
function AIOBFrame_OnLoad()
   AIOB_Register(); -- Slash Commands
   AIOI_Print(AIOB_LOADED);
   this:RegisterEvent("UNIT_NAME_UPDATE");
   this:RegisterEvent("PLAYER_ENTERING_WORLD");
   this:RegisterEvent("VARIABLES_LOADED");
   this:RegisterEvent("BAG_UPDATE");
   this:RegisterEvent("ITEM_LOCK_CHANGED");
   this:RegisterEvent("UPDATE_INVENTORY_ALERTS");
   this:RegisterEvent("BANKFRAME_OPENED");
   this:RegisterEvent("BANKFRAME_CLOSED");
   this:RegisterEvent("PLAYERBANKSLOTS_CHANGED");
   this:RegisterEvent("PLAYERBANKBAGSLOTS_CHANGED");
   this:RegisterEvent("PLAYER_MONEY");
   tinsert(UISpecialFrames, "AIOBFrame"); -- Esc Closes AIOB
end

function AIOB_Register()
   SlashCmdList["AIOBSLASHMAIN"] = AIOB_ChatCommandHandler;
   SLASH_AIOBSLASHMAIN1 = "/AIOB";
end
-- End Load }}}
-- Confirm Dialog for buying bag slot {{{
function AIOB_RegisterConfirm()
   PurchaseSlot_Saved = PurchaseSlot;
   PurchaseSlot = AIOB_PurchaseSlot;
   StaticPopupDialogs["PURCHASE_BANKBAG"] = {
      text = TEXT(AIOB_PURCHASE_CONFIRM_S),
      button1 = TEXT(ACCEPT),
      button2 = TEXT(CANCEL),
      OnAccept = function()
         PurchaseSlot_Saved();
      end,
      showAlert = 1,
      timeout = 0,
   };
end

function AIOB_PurchaseSlot()
   if not StaticPopupDialogs["PURCHASE_BANKBAG"] then
      return;
   end
   local cost = GetBankSlotCost();
   if cost < 10000 then
      StaticPopupDialogs["PURCHASE_BANKBAG"]["text"] = format(AIOB_PURCHASE_CONFIRM_S,(cost/100));
   else
      StaticPopupDialogs["PURCHASE_BANKBAG"]["text"] = format(AIOB_PURCHASE_CONFIRM_G,(cost/10000));
   end
   StaticPopup_Show("PURCHASE_BANKBAG");
end
-- End buy bag slot config }}}
-- Register with MyAddons
function AIOB_MyAddonsRegister()
   if (myAddOnsFrame) then
      myAddOnsList.AIOB = {
         name = AIOB_MYADDON_NAME,
         description = AIOB_MYADDON_DESCRIPTION,
         version = AIOB_VERSION,
         category = MYADDONS_CATEGORY_INVENTORY,
         frame = "AIOBFrame",
         optionsframe = "AIOBConfigFrame"
      };
   end
end

function AIOBFrame_OnEvent(event)
   if ( event == "VARIABLES_LOADED" ) then
      AIOI_DEBUG("AIOB: Variables_loaded event");
      AIOB_Loaded = 1;
      AIOB_MyAddonsRegister();
      --	AIOB_InitializeProfile();
      if not BankBuyFrame then
         AIOB_RegisterConfirm();
      end
   end
   if (not AIOB_Loaded) then
      return;
   end

   if ( event == "UNIT_NAME_UPDATE" and arg1 == "player" and UnitName("player") ~= UNKNOWNOBJECT and bankPlayer == nil ) then
      AIOI_DEBUG("AIOB: UNIT_NAME_UPDATE event");
      PlayerName = UnitName("player").."|"..AIOI_Trim(GetCVar("realmName"));
      AIOB_setPlayerName(PlayerName)
      bankPlayer = AIOB_GetPlayer(PlayerName);
      AIOB_setBankPlayer(bankPlayer)
   elseif ( event == "PLAYER_ENTERING_WORLD") then
      AIOI_DEBUG("AIOB: PLAYER_ENTERING_WORLD event");
     if (PlayerName == nil) then
      PlayerName = UnitName("player").."|"..AIOI_Trim(GetCVar("realmName"));
      AIOB_setPlayerName(PlayerName)
      bankPlayer = AIOB_GetPlayer(PlayerName);
      AIOB_setBankPlayer(bankPlayer)
     end
      AIOB_InitializeProfile();
      AIOB_SaveMoney();
   elseif ( event == "BAG_UPDATE" ) then
      AIOI_DEBUG("AIOB: BAG_UPDATE event");
      if AtBank and arg1 >=5 and arg1 <=11 then
         AIOBFrame_SaveItems();
      end
   elseif (event == "PLAYERBANKSLOTS_CHANGED" or event=="PLAYERBANKBAGSLOTS_CHANGED") then
      AIOBFrame_SaveItems();
   elseif ( event == "ITEM_LOCK_CHANGED" or  event == "UPDATE_INVENTORY_ALERTS" ) then
      if ( AtBank ) then
         AIOBFrame_PopulateFrame();
      end
   end
   if (event == "BANKFRAME_OPENED") then
      AtBank = true;
      SetPortraitTexture(AIOBFramePortrait, "npc");
      AIOBFrameAtBankText:Show();
      OpenBackpack(); -- Open Backpack at Bank
      bankPlayer = AIOB_GetPlayer(PlayerName);
      AIOBFrame_SaveItems();
      AIOBFramePurchaseButton:Enable();
      if AIOBReplaceBank == 1 then
         OpenAIOBFrame();
      end
   elseif (event == "BANKFRAME_CLOSED") then
      AtBank = false;
      AIOBFramePortrait:SetTexture("Interface\\Addons\\AllInOneInventory\\Skin\\AIOBPortait");
      AIOBFrameAtBankText:Hide();
      AIOBFramePurchaseButton:Disable();
      CloseBackpack(); -- Close Backpack when leaving
      if AIOBReplaceBank == 1 then
         if AIOBFreeze == 0 then
            CloseAIOBFrame();
         end
      end
      if StackSplitFrame:IsVisible() then
         StackSplitFrame:Hide();
      end
   elseif (event == "PLAYER_MONEY" ) then
      AIOB_SaveMoney();
      return;
   end
end

function AIOB_HighlightBag(bagID, bagName, isItem)
   if AIOBHighlightBags == 0 and isItem then
      return;
   end
   if AIOBHighlightItems == 0 and (not isItem) then
      return;
   end

   if isItem then
      if bagID > -1 then
         getglobal("AIOBFrameBag"..bagID):LockHighlight();
      end
   end

   if(bankPlayer[bagName])then
      local i, found;
      for i=1, AIOB_MAX_ID do
         local itemButton = getglobal("AIOBFrameItem"..i);
         if not itemButton:IsVisible() then
            break;
         end
         if itemButton.bagIndex == bagID then
            found = true;
            itemButton:LockHighlight();
         else
            if found then
               break;
            end
         end
      end
   end
end

function AIOBFrame_UpdateCooldown(button)
   if (not button.bagIndex) or (not button.itemIndex) then
      return;
   end
  local cooldown = getglobal(button:GetName().."Cooldown");
  local start, duration, enable = GetContainerItemCooldown(button.bagIndex, button.itemIndex);
  CooldownFrame_SetTimer(cooldown, start, duration, enable);
  if ( duration > 0 and enable == 0 ) then
    SetItemButtonTextureVertexColor(button, 0.4, 0.4, 0.4);
  end
end

-- == End Event Handling ==

function AIOBFrame_SetColumns(col)
   if ( type(col) ~= "number" ) then
      col = tonumber(col);
   end
   if ( col == nil ) then
      AIOI_DEBUG("Cols Error");
   else
   if ( ( col >= AIOB_COLUMNS_MIN ) and ( col <= AIOB_COLUMNS_MAX ) ) then
      AIOBColumns = col;
      bankPlayer.Columns = AIOBColumns;
      AIOBFrame_UpdateLook(getglobal("AIOBFrame"), AIOB_GetBagsTotalSlots());
   end
end
end

function AIOBFrame_GetAppropriateHeight(rows)
   local height = AIOB_BASE_HEIGHT + ( AIOB_ROW_HEIGHT * (AIOBBagView - 1 + rows ));
   if AIOBShowPlayers == 0 and AIOBGraphics == 0 then
      height = height - AIOB_ROW_HEIGHT;
   end
   return height;
end

function AIOBFrame_GetAppropriateWidth(cols)
   return AIOB_BASE_WIDTH + ( AIOB_COL_WIDTH * cols );
end

function AIOBTitle_Update()
   if AIOBCash == 1 then
      AIOB_TotalMoneyText:Show();
      AIOB_MoneyFrameTotal:Show();
   else
      AIOB_TotalMoneyText:Hide();
      AIOB_MoneyFrameTotal:Hide();
   end

   AIOBSlots_Update()

   if ( bankPlayerName ) then
      local playername = AIOB_Split(bankPlayerName, "|");
      if ( AIOBColumns >= 9 ) then
         AIOBFrameName:SetText(format(AIOB_FRAME_PLAYERANDREGION, playername[1], playername[2]));
      else
         AIOBFrameName:SetText(format(AIOB_FRAME_PLAYERONLY, playername[1]));
      end
      AIOBFrameName:SetTextColor(1.0, 1.0, 1.0);
   end
end

function AIOBSlots_Update()
   local i, j, totalSlots, takenSlots = 0, 0, 0, 0;
   totalSlots = AIOB_GetBagsTotalSlots();
   -- Need to calculate Free slots.
   if bankPlayer and bankPlayer["Bank"] then
      for i = 1, 28 do
         if bankPlayer["Bank"][i] then
            takenSlots = takenSlots + 1;
         end
      end
      for i = 5, 11 do
         if bankPlayer["Bag"..i] and AIOI_GetItemSlots(bankPlayer["Bag"..i]) then
            for j = 1, AIOI_GetItemSlots(bankPlayer["Bag"..i]) do
               if bankPlayer["Bag"..i][j] then
                  takenSlots = takenSlots + 1;
               end
            end
         end
      end
   end

   if AIOBCount == 0 then
      AIOBFrameSlots:Show();
      AIOBFrameSlots:SetText(format(AIOB_FRAME_SLOTS, (totalSlots-takenSlots),totalSlots));
   elseif AIOBCount == 1 then
      AIOBFrameSlots:Show();
      AIOBFrameSlots:SetText(format(AIOB_FRAME_SLOTS, (takenSlots),totalSlots));
   else
      AIOBFrameSlots:Hide();
   end
end

function AIOB_UpdateTotalMoney()
   local totalMoney = 0;
   for key, value in pairs(AIOBProfile) do
      local thisRealmPlayers = AIOB_Split(key, "|")[2];
      if AIOBAllRealms == 1 or thisRealmPlayers == AIOI_Trim(GetCVar("realmName")) then
         if ( AIOBProfile[key].money ) then
            totalMoney = totalMoney + AIOBProfile[key].money;
         end
      end
   end
   if AIOBColumns < 8 then
      totalMoney = math.floor(totalMoney / 10000) * 10000;
   end
   MoneyFrame_Update("AIOB_MoneyFrameTotal", totalMoney or 0);
end

function AIOB_UpdateBagCost(bags)
   if not bags then
      bags=0;
   end
   if bags < 7 then
      AIOBFramePurchaseInfo:Show();
      local cost = AIOB_GetBankSlotCost(bags);
      MoneyFrame_Update("AIOBFrameDetailMoneyFrame", cost);
      if ( bankPlayer and bankPlayer["money"] and bankPlayer["money"] >= cost ) then
         SetMoneyFrameColor("AIOBFrameDetailMoneyFrame", 1.0, 1.0, 1.0);
      else
         SetMoneyFrameColor("AIOBFrameDetailMoneyFrame", 1.0, 0.1, 0.1)
      end
   else
      -- Hide frame
      AIOBFramePurchaseInfo:Hide();
   end
end

function AIOBFrame_UpdateLookIfNeeded()
   local slots = AIOB_GetBagsTotalSlots();
   if ( ( not AIOBFrame.size ) or ( slots ~= AIOBFrame.size ) ) then
      AIOBFrame_UpdateLook(getglobal("AIOBFrame"), slots);
   end
end

function AIOBFrame_UpdateLook(frame, frameSize)

   AIOB_ShowButtons();  -- Show or Hide Buttons

   frame.size = frameSize;
   local name = frame:GetName();
   local columns = AIOBColumns;

   local rows = ceil(frame.size / columns);
   local height = AIOBFrame_GetAppropriateHeight(rows);
   frame:SetHeight(height);

   local width = AIOBFrame_GetAppropriateWidth(columns);
   frame:SetWidth(width);

   AIOBTitle_Update();
   if AIOBShowPlayers ==1 then
      AIOBDropDown:Show();
      AIOB_AllRealms_Check:Show();
      --MYINVENTORY_BASE_HEIGHT = MYINVENTORY_BASE_HEIGHT - MYINVENTORY_ITEM_OFFSET_Y;
   else
      AIOBDropDown:Hide();
      AIOB_AllRealms_Check:Hide();
   end
   for j=5,11 do
      local bagButton=getglobal("AIOBFrameBag"..j);
      bagButton:ClearAllPoints();
      if j == 5 then
         bagButton:SetPoint("TOPLEFT", "AIOBBagButtonsBar", "TOPLEFT", 0, 0);
      else
         bagButton:SetPoint("TOPLEFT", "AIOBFrameBag"..(j-1), "TOPLEFT", AIOB_ITEM_OFFSET_X, 0);
      end
      bagButton:Show();
   end
   local First_Y;
      First_Y = AIOB_FIRST_ITEM_OFFSET_Y;
   if AIOBBagView == 1 then
      First_Y = AIOB_FIRST_ITEM_OFFSET_Y + AIOB_ITEM_OFFSET_Y;
      AIOBBagButtonsBar:Show();
   else
      First_Y = AIOB_FIRST_ITEM_OFFSET_Y;
      AIOBBagButtonsBar:Hide();
   end
   if (AIOBShowPlayers == 0 and AIOBGraphics == 0 ) then
      AIOI_DEBUG("move up");
      First_Y = First_Y - AIOB_ITEM_OFFSET_Y;
      AIOBBagButtonsBar:ClearAllPoints();
      AIOBBagButtonsBar:SetPoint("TOP", "AIOBFrame", "TOP", 0, -28);
   else
      AIOBBagButtonsBar:ClearAllPoints();
      AIOBBagButtonsBar:SetPoint("TOP", "AIOBFrame", "TOP", 0, -28-39);

   end
   for j=1, frame.size, 1 do
      local itemButton = getglobal(name.."Item"..j);
      -- Set first button
      itemButton:ClearAllPoints();
      if ( j == 1 ) then
         itemButton:SetPoint("TOPLEFT", name, "TOPLEFT", AIOB_FIRST_ITEM_OFFSET_X, First_Y);
      else
         if ( mod((j-1), columns) == 0 ) then
            itemButton:SetPoint("TOPLEFT", name.."Item"..(j - columns), "TOPLEFT", 0, AIOB_ITEM_OFFSET_Y);
         else
            itemButton:SetPoint("TOPLEFT", name.."Item"..(j - 1), "TOPLEFT", AIOB_ITEM_OFFSET_X, 0);
         end
      end

      itemButton.readable = readable;
      itemButton:Show();
   end
   local button = nil;
   for i = frame.size+1, AIOB_MAX_ID do
      button = getglobal("AIOBFrameItem"..i);
      if ( button ) then
         button:Hide();
      end
   end
   AIOBFrame_PopulateFrame();
end

function AIOBFrame_PopulateFrame()
   local texture, itemButton, itemCount, itemQuality;
   local bagName, bagMaxIndex, bagNewIndex;
   local buttonIndex = 1;
   local BlankTexture;
   local maxBags;
   if not bankPlayer then
      return;
   end
   if bankPlayer.Bags then
      maxBags = bankPlayer.Bags;
   else
      maxBags = 0;
   end
   AIOB_UpdateBagCost(maxBags);
   _, BlankTexture = GetInventorySlotInfo("Bag0Slot");
   for bagNum = 0, maxBags do
      if bagNum == 0 then -- Is it the bank?
         bagName = "Bank";
         bagNewIndex = BANK_CONTAINER;
         bagMaxIndex = 28;
      else
         bagName = "Bag"..(4+bagNum);
         bagNewIndex = (4+bagNum);
         local bagButton = getglobal("AIOBFrameBag"..(bagNewIndex));
         SetItemButtonNormalTextureVertexColor(bagButton, 1.0,1.0,1.0);
         SetItemButtonTextureVertexColor(bagButton, 1.0,1.0,1.0);
         if bankPlayer[bagName] and AIOI_GetItemSlots(bankPlayer[bagName]) then
            bagMaxIndex = AIOI_GetItemSlots(bankPlayer[bagName]);
            SetItemButtonTexture(bagButton, "Interface\\Icons\\"..AIOI_GetItemTexture(bankPlayer[bagName]));
         else
            bagMaxIndex = 0;
            SetItemButtonTexture(bagButton, BlankTexture);
         end
      end
      for itemIndex = 1,bagMaxIndex do
         itemButton = getglobal("AIOBFrameItem"..buttonIndex);
         buttonIndex = buttonIndex + 1;
         if(bankPlayer and bankPlayer[bagName] and bankPlayer[bagName][itemIndex]) then
            texture = "Interface\\Icons\\"..AIOI_GetItemTexture(bankPlayer[bagName][itemIndex]);
            itemCount = AIOI_GetItemCount(bankPlayer[bagName][itemIndex]);
         else
            texture = nil;
            itemCount = 0;
         end
         if(itemButton) then
            local locked;
            if AtBank then
               texture, itemCount, locked, itemQuality, itemReadable = GetContainerItemInfo(bagNewIndex, itemIndex);
               AIOBFrame_UpdateCooldown(itemButton);
            else
               locked = nil;
            end
            if bankPlayer[bagName] and bankPlayer[bagName][itemIndex] and AIOI_GetCoolDownDuration(bankPlayer[bagName][itemIndex]) then
               local cooldown = getglobal(itemButton:GetName().."Cooldown");
               local cooldownInfo = AIOI_GetCoolDownDuration(bankPlayer[bagName][itemIndex]);
               if cooldownInfo and AIOI_GetCoolDownEnabled(cooldownInfo) then
                  local start, duration, enable = AIOI_GetCoolDownStart(cooldownInfo), AIOI_GetCoolDownDuration(cooldownInfo), AIOI_GetCoolDownEnabled(cooldownInfo);
                  if duration > 0 then
                     CooldownFrame_SetTimer(cooldown, start, duration, enable);
                  else
                     cooldown:Hide();
                  end
               else
                  cooldown:Hide();
               end
            end
            if bankPlayer[bagName] and bankPlayer[bagName][itemIndex] and AIOB_GetItemQuality(bankPlayer[bagName][itemIndex]) then
               AIOI_UpdateBorder(itemButton, AIOB_GetItemQuality(bankPlayer[bagName][itemIndex]));
            else
               AIOI_UpdateBorder(itemButton, itemQuality, bagNum);
            end
            SetItemButtonTexture(itemButton, texture);
            SetItemButtonCount(itemButton, itemCount);
            SetItemButtonDesaturated(itemButton, locked, 0.5, 0.5, 0.5);
            itemButton.bagIndex = bagNewIndex;
            itemButton.itemIndex= itemIndex;
         end
      end
   end
   for bagNum = 5+maxBags, 11 do
      local bagButton = getglobal("AIOBFrameBag"..(bagNum));
      SetItemButtonNormalTextureVertexColor(bagButton, 1.0,0.1,0.1);
      SetItemButtonTextureVertexColor(bagButton, 1.0,0.1,0.1);
      SetItemButtonTexture(bagButton, BlankTexture);
   end
   if ( bankPlayer and  AIOB_GetMoney(bankPlayer) ) then
      MoneyFrame_Update("AIOB_MoneyFrame", AIOB_GetMoney(bankPlayer));
      AIOB_MoneyFrame:Show();
   else
      AIOB_MoneyFrame:Hide();
   end
   AIOB_UpdateTotalMoney();
   AIOBFrame_UpdateLookIfNeeded();
end

function AIOB_SetScale(scale) --
   local pScale = tonumber(AIOBFrame:GetParent():GetScale());
   if ( pScale == nil ) then
      pScale = tonumber(GetCVar("uiscale"));
      if not pScale then
         pScale = 1;
         AIOI_DEBUG("Scale Error")
      end
   end

   AIOBFrame:SetScale(pScale * tonumber(scale));
   AIOB_Toggle_Option("Scale", scale, 1);
end


-- == Viewing other peoples banks ==
function AIOB_UserDropDown_GetValue()
   if ( bankPlayerName ) then
      return bankPlayerName;
   else
      return (UnitName("player").."|"..AIOI_Trim(GetCVar("realmName")));
   end
end
function AIOB_UserDropDown_OnLoad()
   UIDropDownMenu_Initialize(this, AIOB_UserDropDown_Initialize);
   UIDropDownMenu_SetSelectedValue(this, AIOB_UserDropDown_GetValue());
   AIOBDropDown.tooltip = "You are viewing this player's bank contents.";
   UIDropDownMenu_SetWidth(140, AIOBDropDown);
   -- OptionsFrame_EnableDropDown(AIOBDropDown);
end

function AIOB_UserDropDown_OnClick()
   if ( not bankPlayer ) then
      return;
   end
   if AtBank then
      CloseBankFrame();
      OpenAIOBFrame();
   end
   -- UIDropDownMenu_SetSelectedValue(AIOBDropDown, this.value);
   if ( this.value ) then
      bankPlayer = AIOB_GetPlayer(this.value);
      AIOB_setBankPlayer(bankPlayer)
   end
   AIOBFrame_PopulateFrame();
end

function AIOB_UserDropDown_Initialize()
   local selectedValue = UIDropDownMenu_GetSelectedValue(AIOBDropDown);
   local info;
   for key, value in pairs(AIOBProfile) do
      local thisRealmPlayers = AIOB_Split(key, "|")[2];
      if ( table.getn(AIOBProfile[key]) > 0 or AIOBProfile[key].money ) then
         if (AIOBAllRealms == 1 or thisRealmPlayers == AIOI_Trim(GetCVar("realmName")) ) then
            info = {};
            info.text = AIOB_Split(key,"|")[1].." of "..AIOB_Split(key,"|")[2];
            info.value = key;
            info.func = AIOB_UserDropDown_OnClick;
            if ( selectedValue == info.value ) then
               info.checked = 1;
            else
               info.checked = nil;
            end
            UIDropDownMenu_AddButton(info);
         end
      end
   end
end

function AIOB_ShowAllRealms_Check_OnClick()
      if ( AIOBAllRealms == 0 ) then
         AIOBAllRealms = 1;
      else
         AIOBAllRealms = 0;
      end
      AIOB_UpdateTotalMoney();
end

function AIOB_ShowAllRealms_Check_OnShow()
   if ( AIOBAllRealms == 1 ) then
      this.checked = 1;
   else
      this.checked = nil;
   end
   OptionsFrame_EnableCheckBox(this);
   this:SetChecked(this.checked);
   this.tooltipText = "Check to show all saved characters, regardless of realm.";
end


-- === Toggle Functions ===
-- All toggling of options
function ToggleAIOBFrame()
   if ( AIOBFrame:IsVisible() ) then
      CloseAIOBFrame();
   else
      OpenAIOBFrame();
   end
end
function CloseAIOBFrame()
   if AtBank then
      CloseBankFrame();
   end
   if ( AIOBFrame:IsVisible() ) then
      HideUIPanel(AIOBFrame);
   end
end
function OpenAIOBFrame()
   AIOBFrame_UpdateLookIfNeeded();
   ShowUIPanel(AIOBFrame, 1);
end

function AIOB_Toggle_Option(option, value, quiet)
   if value == nil then
      if getglobal("AIOB"..option) == 1 then
         value = 0;
      else
         value = 1;
      end
   end
   setglobal("AIOB"..option, value);
   AIOBProfile[PlayerName][option] = value;
   if not quiet then
      local chat_message;
      local globalName = "AIOB_CHAT_"..string.upper(option);
      if value == 0 then
         globalName = globalName.."OFF";
      else
         globalName = globalName.."ON";
      end
      chat_message = getglobal(globalName);
      if ( chat_message ) then
         AIOI_Print(chat_message);
      else
         AIOI_DEBUG("ERROR: No global "..globalName);
      end
   end
   if AIOBGraphics == 1 then
      AIOB_OptionsFrame_CheckButton_Background:Hide();
   else
      AIOB_OptionsFrame_CheckButton_Background:Show();
   end
   if option == "ReplaceBank" then
      AIOB_SetReplaceBank();
   elseif option == "ShowPlayers" or option == "BagView" then
      AIOBFrame_UpdateLook(getglobal("AIOBFrame"),AIOB_GetBagsTotalSlots());
   elseif option == "Graphics" or option == "Background" then
      AIOB_SetGraphics();
      AIOBFrame_UpdateLook(getglobal("AIOBFrame"),AIOB_GetBagsTotalSlots());
   elseif option == "Freeze" then
      AIOB_SetFreeze();
   end
end

function AIOB_ShowButtons()
   if AIOBButtons == 1 then
      AIOBFrameCloseButton:Show();
      AIOBFrameLockButton:Show();
      AIOBFrameHideBagsButton:Show();
   else
      AIOBFrameCloseButton:Hide();
      AIOBFrameLockButton:Hide();
      AIOBFrameHideBagsButton:Hide();
   end
   -- always show the close button for blizz style graphics.
   if AIOBGraphics == 1 then
      AIOBFrameCloseButton:Show();
   end
end

function AIOB_SetGraphics()
   local pScale = tonumber(AIOBFrame:GetParent():GetScale());
   local scale = AIOBScale;
   if ( pScale == nil ) then
      pScale = tonumber(GetCVar("uiscale"));
      if not pScale then
         pScale = 1;
         AIOI_DEBUG("Scale Error")
      end
   end

   if AIOBGraphics == 1 then
      AIOBFrame:SetBackdropColor(0,0,0,0);
      AIOBFrame:SetBackdropBorderColor(0,0,0,0);

      AIOBFramePortrait:Show();
      AIOBFrameTextureTopLeft:Show();
      AIOBFrameTextureTopCenter:Show();
      AIOBFrameTextureTopRight:Show();
      AIOBFrameTextureLeft:Show();
      AIOBFrameTextureCenter:Show();
      AIOBFrameTextureRight:Show();
      AIOBFrameTextureBottomLeft:Show();
      AIOBFrameTextureBottomCenter:Show();
      AIOBFrameTextureBottomRight:Show();

      AIOBFrameName:ClearAllPoints();
      AIOBFrameName:SetPoint("TOPLEFT", "AIOBFrame", "TOPLEFT", 70, -8);
      AIOBFrameCloseButton:ClearAllPoints();
      AIOBFrameCloseButton:SetPoint("TOPRIGHT", "AIOBFrame", "TOPRIGHT", 10, 0);
   else
      if AIOBBackground==1 then
         AIOBFrame:SetBackdropColor(0,0,0,1.0);
         AIOBFrame:SetBackdropBorderColor(1,1,1,1.0);
      else
         AIOBFrame:SetBackdropColor(0,0,0,0);
         AIOBFrame:SetBackdropBorderColor(1,1,1,0);
      end

      AIOBFramePortrait:Hide();
      AIOBFrameTextureTopLeft:Hide();
      AIOBFrameTextureTopCenter:Hide();
      AIOBFrameTextureTopRight:Hide();
      AIOBFrameTextureLeft:Hide();
      AIOBFrameTextureCenter:Hide();
      AIOBFrameTextureRight:Hide();
      AIOBFrameTextureBottomLeft:Hide();
      AIOBFrameTextureBottomCenter:Hide();
      AIOBFrameTextureBottomRight:Hide();

      AIOBFrameName:ClearAllPoints();
      AIOBFrameName:SetPoint("TOPLEFT", "AIOBFrame", "TOPLEFT", 5, -6);
      AIOBFrameCloseButton:ClearAllPoints();
      AIOBFrameCloseButton:SetPoint("TOPRIGHT", "AIOBFrame", "TOPRIGHT", 2, 2);
   end
   AIOBFrame:SetScale(pScale * tonumber(scale));
end


function AIOB_SetFreeze()
   if AIOBFreeze == 1 then
      AIOBFreezeNormalTexture:SetTexture("Interface\\AddOns\\AllInOneInventory\\Skin\\LockButton-Locked-Up");
   else
      AIOBFreezeNormalTexture:SetTexture("Interface\\AddOns\\AllInOneInventory\\Skin\\LockButton-Unlocked-Up");
   end
end

-- SetReplaceBank: Sets if AIOB replaces the official Bank frame
-- Unhooks the Official blizzard frome from the opened and closed events
function AIOB_SetReplaceBank()
   if BankFrame_Saved == nil then
      BankFrame_Saved = getglobal("BankFrame");
   end
   if ( AIOBReplaceBank == 0 ) then
      BankFrame_Saved:RegisterEvent("BANKFRAME_OPENED");
      BankFrame_Saved:RegisterEvent("BANKFRAME_CLOSED");
      setglobal("BankFrame", BankFrame_Saved);
      BankFrame_Saved = nil;
   else
      if BankFrame_Saved:IsVisible() then
         BankFrame_Saved:Hide();
      end
      BankFrame_Saved:UnregisterEvent("BANKFRAME_OPENED");
      BankFrame_Saved:UnregisterEvent("BANKFRAME_CLOSED");
      setglobal("BankFrame", AIOBFrameAtBankText);
   end
end
-- == End Toggle Functions ==

function AIOB_SetScale(scale) -- {{{
   if ( type(scale) ~= "number" ) then
      scale = tonumber(scale);
   end
   if ( scale == nil ) then
      AIOI_DEBUG("Scale Error")
   else
      if ( ( scale >= 0.5 ) and ( scale <= 2 ) ) then
         local pScale = tonumber(AIOBFrame:GetParent():GetScale());
         if ( pScale == nil ) then
            pScale = tonumber(GetCVar("uiscale"));
            if not pScale then
               pScale = 1;
               AIOI_DEBUG("Scale Error")
            end
         end
         AIOBFrame:SetScale(pScale * tonumber(scale));
         AIOBScale = scale;
         bankPlayer.Scale = AIOBScale;
      end
   end
end

function AIOB_GetAtBank()
   return AtBank;
end
