-- -----------------------------------------------------------------
-- File: AllInOneBank_Config.lua
--
-- Purpose: Functions for AIOB Config window and options.
--
-- Author: Ramble
--
-- Credits:
--   Starven, for MyInventory
--   Kaitlin, for BankItems
--   Sarf, for the original concept of AllInOneInventory
-- -----------------------------------------------------------------

-- == Slash Handlers ==
-- ChatCommandHandler: Slash commands are in here
function AIOB_ChatCommandHandler(msg)

   if ( ( not msg ) or ( strlen(msg) <= 0 ) ) then
      AIOI_Print(AIOB_CHAT_COMMAND_USAGE);
      AIOBOptionsFrame:Show();
      return;
   end

   local commandName, params = AIOI_Extract_NextParameter(msg);

   if ( ( commandName ) and ( strlen(commandName) > 0 ) ) then
      commandName = string.lower(commandName);
   else
      commandName = "";
   end

   if ( (strfind(commandName, "show")) or (strfind(commandName, "toggle")) ) then
      ToggleAIOBFrame();
   elseif ( (strfind(commandName, "freeze")) or (strfind(commandName, "unfreeze")) ) then
      AIOB_Toggle_Option("Freeze");
   elseif ( (strfind(commandName, "replacebank")) or (strfind(commandName, "replace"))) then
      AIOB_Toggle_Option("ReplaceBank");
   elseif strfind(commandName, "highlightbags") then
      AIOB_Toggle_Option("HighlightBags");
   elseif strfind(commandName, "highlightitems") then
      AIOB_Toggle_Option("HighlightItems");
   elseif strfind(commandName, "graphic") then
      AIOB_Toggle_Option("Graphics");
   elseif strfind(commandName, "back") then
      AIOB_Toggle_Option("Background");
   elseif strfind(commandName, "player") then
      AIOB_Toggle_Option("ShowPlayers");
   elseif ( (strfind(commandName, "cols")) or (strfind(commandName, "column")) ) then
      cols, params = AIOI_Extract_NextParameter(params);
      cols = tonumber(cols);
      AIOBFrame_SetColumns(cols);
   elseif ( (strfind(commandName, "reset")) or (strfind(commandName, "init")) ) then
      AIOBProfile[UnitName("player")] = nil;
      AIOB_InitializeProfile();
   elseif (strfind(commandName, "config")) then
      AIOBConfigFrame:Show();
   else
      AIOI_Print(AIOB_CHAT_COMMAND_USAGE);
      return;
   end
end
-- == End Slash Handlers ==


--All Functions that deal directly with AIOBConfig Frame items should go here
function AIOBConfig_OnShow()
   -- AIOBConfigReplaceCheck:SetChecked(AIOBReplaceBank);
   -- AIOBConfigFreezeCheck:SetChecked(AIOBFreeze);
   -- AIOBConfigColumns:SetText(AIOBColumns);
   -- AIOBConfigHighlightItemsCheck:SetChecked(AIOBHighlightItems);
   -- AIOBConfigHighlightBagsCheck:SetChecked(AIOBHighlightBags);
   -- AIOBConfigHighlightItemsCheck:SetChecked(AIOBHighlightItems);
   -- AIOBConfigGraphicsCheck:SetChecked(AIOBGraphics);
   -- AIOBConfigBackgroundCheck:SetChecked(AIOBBackground);
end

function AIOBConfig_OnHide()
   if AIOBConfigReplaceCheck:GetChecked() then
      AIOB_Toggle_Option("ReplaceBank", 1,1);
   else
      AIOB_Toggle_Option("ReplaceBank", 0,1);
   end
   if AIOBConfigFreezeCheck:GetChecked() then
      AIOB_Toggle_Option("Freeze", 1,1);
   else
      AIOB_Toggle_Option("Freeze", 0,1);
   end
   local cols =  tonumber(AIOBConfigColumns:GetText());
   if cols and cols > 5 and cols < 19 then
      AIOBFrame_SetColumns(cols);
   end
   if AIOBConfigHighlightItemsCheck:GetChecked() then
      AIOB_Toggle_Option("HighlightItems",1,1);
   else
      AIOB_Toggle_Option("HighlightItems",0,1);
   end
   if AIOBConfigHighlightBagsCheck:GetChecked() then
      AIOB_Toggle_Option("HighlightBags",1,1);
   else
      AIOB_Toggle_Option("HighlightBags",0,1);
   end
   if AIOBConfigGraphicsCheck:GetChecked() then
      AIOB_Toggle_Option("Graphics",1,1);
   else
      AIOB_Toggle_Option("Graphics",0,1);
   end
   if AIOBConfigBackgroundCheck:GetChecked() then
      AIOB_Toggle_Option("Background",1,1);
   else
      AIOB_Toggle_Option("Background",0,1);
   end
end

-- -----------------------------------------------------------------
-- File: AIOI_Options.lua
--
-- Purpose: Functions for AIOI WoW Window.
--
-- Author: Arnvid Karstad - ievil - Asys @ Turalyon_EU
--         Most code exists in it's original form in this file from Svarten and Ramble.
--
-- Credits:
--   Sarf, for the original concept.
--   Svarten and Ramble for bringing the original MyInventory to life
--   Roger V, for the updated ContainerFrameItemButtonTemplate fix
-- -----------------------------------------------------------------

function AIOBOptionsFrame_OnLoad()
	tinsert(UISpecialFrames, "AIOBOptionsFrame"); -- Esc Closes Inventory
	UIPanelWindows["AIOBOptionsFrame"] = {area = "center", pushable = 0};

end

function AIOBOptionsFrame_OnShow()
	-- Load Checks
	AIOB_OptionsFrame_CheckButton_ReplaceBank:SetChecked(AIOBReplaceBank);
	AIOB_OptionsFrame_CheckButton_Graphics:SetChecked(AIOBGraphics);
	AIOB_OptionsFrame_CheckButton_Background:SetChecked(AIOBBackground);
	AIOB_OptionsFrame_CheckButton_HighlightBags:SetChecked(AIOBHighlightBags);
	AIOB_OptionsFrame_CheckButton_HighlightItems:SetChecked(AIOBHighlightItems);
	AIOB_OptionsFrame_CheckButton_Cash:SetChecked(AIOBCash);
	AIOB_OptionsFrame_CheckButton_Buttons:SetChecked(AIOBButtons);
	AIOB_OptionsFrame_CheckButton_BagView:SetChecked(AIOBBagView);
	AIOB_OptionsFrame_CheckButton_Freeze:SetChecked(AIOBFreeze);

   if AIOBCount == 1 then
		AIOB_OptionsFrame_CheckButton_CountUsed:SetChecked(1);
		AIOB_OptionsFrame_CheckButton_CountFree:SetChecked(0);
		AIOB_OptionsFrame_CheckButton_CountOff:SetChecked(0);

   elseif AIOBCount == 0 then
		AIOB_OptionsFrame_CheckButton_CountUsed:SetChecked(0);
		AIOB_OptionsFrame_CheckButton_CountFree:SetChecked(1);
		AIOB_OptionsFrame_CheckButton_CountOff:SetChecked(0);

   else
		AIOB_OptionsFrame_CheckButton_CountUsed:SetChecked(0);
		AIOB_OptionsFrame_CheckButton_CountFree:SetChecked(0);
		AIOB_OptionsFrame_CheckButton_CountOff:SetChecked(1);
	end

   if AIOBGraphics == 1 then
      AIOB_OptionsFrame_CheckButton_Background:Hide();
   else
      AIOB_OptionsFrame_CheckButton_Background:Show();
   end

	AIOB_OptionsFrame_Slider_Cols:SetMinMaxValues(AIOB_COLUMNS_MIN, AIOB_COLUMNS_MAX);
	AIOB_OptionsFrame_Slider_Cols:SetValueStep(1);
	AIOB_OptionsFrame_Slider_ColsText:SetText("Columns");
	AIOB_OptionsFrame_Slider_ColsLow:SetText(AIOB_COLUMNS_MIN);
	AIOB_OptionsFrame_Slider_ColsHigh:SetText(AIOB_COLUMNS_MAX);

	AIOB_OptionsFrame_Slider_Scale:SetMinMaxValues(0.5, 1.5);
	AIOB_OptionsFrame_Slider_Scale:SetValueStep(0.01);
	AIOB_OptionsFrame_Slider_ScaleText:SetText("Scale");
	AIOB_OptionsFrame_Slider_ScaleLow:SetText("0.5");
	AIOB_OptionsFrame_Slider_ScaleHigh:SetText("1.5");
	AIOB_OptionsFrame_Slider_Cols:SetValue(AIOBColumns);
	AIOB_OptionsFrame_Slider_Scale:SetValue(AIOBScale);
end

function AIOB_OptionCheck_OnLoad()
	local option = strsub(this:GetName(), 31);
	-- Set Check button text.
	local label = getglobal(this:GetName().."Text");
	local optiontext = "AIOB_CHECKTEXT_"..string.upper(option);
	if getglobal(optiontext) then
		label:SetText(getglobal(optiontext));
	else
		label:SetText(option);
	end
	-- Set tooltip text :
	local optiontip = getglobal("AIOB_CHECKTIP_"..string.upper(option));
	if optiontip then
		this.tooltipText = optiontip;
	end
end

function AIOB_OptionCheck_OnClick()
	local value;
	if ( this:GetChecked() ) then
		PlaySound("igMainMenuOptionCheckBoxOff");
		value = 1;
	else
		PlaySound("igMainMenuOptionCheckBoxOn");
		value = 0;
	end
	--Set Option
	local option = strsub(this:GetName(), 31);
	AIOB_Toggle_Option(option, value, 1);
	AIOBFrame_UpdateLook(getglobal("AIOBFrame"),AIOB_GetBagsTotalSlots());
end

function AIOB_CountCheck_OnClick()
	local value;
	this:SetChecked(1);
	if AIOB_OptionsFrame_CheckButton_CountUsed ~= this then
		AIOB_OptionsFrame_CheckButton_CountUsed:SetChecked(0);
	else
		value = 1
	end
	if AIOB_OptionsFrame_CheckButton_CountFree ~= this then
		AIOB_OptionsFrame_CheckButton_CountFree:SetChecked(0);
	else
		value = 0
	end
	if AIOB_OptionsFrame_CheckButton_CountOff ~= this then
		AIOB_OptionsFrame_CheckButton_CountOff:SetChecked(0);
	else
		value = -1
	end
	AIOB_Toggle_Option("Count", value, 1);
   AIOBFrame_UpdateLook(getglobal("AIOBFrame"),AIOB_GetBagsTotalSlots());
end

function AIOB_OptionsFrame_Slider_Cols_OnValueChanged()
	if this:GetValue() ~= AIOBColumns then
		AIOBFrame_SetColumns(this:GetValue());
	end
end

function AIOB_OptionsFrame_Slider_Scale_OnValueChanged()
	if this:GetValue() ~= AIOBScale then
		AIOB_SetScale(this:GetValue());
	end
end

function AIOB_Check_OnEnter()
	if ( this.tooltipText ) then
		AIOB_OptionsFrame_Help_Text:SetText(this.tooltipText);
		AIOB_OptionsFrame_Help_Text:SetWidth(350);
	end
end
function AIOB_Check_OnLeave()
	AIOB_OptionsFrame_Help_Text:SetText("");
end

-- END AIOBConfig Frame stuff


