-- -----------------------------------------------------------------
-- File: AIOI.lua
--
-- Purpose: Functions for AIOI WoW Window.
--
-- Credits:
--   Sarf, for the original concept.
--   Svarten and Ramble for bringing the original MyInventory to life
--   Roger V, for the updated ContainerFrameItemButtonTemplate fix
--   Arnvid Karstad - ievil - Asys @ Turalyon_EU (for work on updating and maintaining this)
--   robinson - for work merging ACUI updates & code back to it's roots (back to all in one inventory)
--              also for rolling all bag inventory mods in one (bank + bags). Truly an all in one inventory.
-- Thanks to:
--   Kyzac, Roger V,
-- -----------------------------------------------------------------
AIOIDEBUG                = 0;

-----------------------
-- Saved Configuration
-----------------------
AIOIProfile = {}
local PlayerName = nil;

-----------------------
-- Local Configuration
-----------------------
-- Constants
AIOI_MAX_ID              = 128; -- Backpack (16) + 4* 28 (Core Soulbags thingies)
AIOI_COLUMNS_MIN         =   2;
AIOI_COLUMNS_MAX         =  18;

AIOI_TOP_HEIGHT          =  20;
AIOI_BOTTOM_HEIGHT       =  16;

AIOI_FIRST_ITEM_OFFSET_X =   8;
AIOI_FIRST_ITEM_OFFSET_Y = -28;
AIOI_ITEM_OFFSET_X       =  39;
AIOI_ITEM_OFFSET_Y       = -39;

AIOI_Loaded              = nil;
AIOI_Initialized         = nil;

-- Saved Between sessions:These are also the defaults
AIOIColumns              = 12;

AIOIReplaceBags          = 1;
AIOIBags                 = "11111";
AIOIGraphics             = 1;
AIOIHighlightItems       = 1;
AIOIHighlightBags        = 1;
AIOIFreeze               = 0; -- Keep the window from closing with Close

-- These options are on the Frame Window itself.
AIOILock                 = 0; -- Lock the window in place
AIOIBagView              = 0;

--Visibility
AIOICount                = 1;
AIOIBackground           = 1;
AIOIShowTitle            = 1;
AIOICash                 = 1;
AIOIButtons              = 1;
AIOIScale                = 1;
AIOIReverseStyle         = 0;

-- Hooked functions
AIOI_Saved_ToggleBag                        = nil;
AIOI_Saved_OpenBag                          = nil;
AIOI_Saved_CloseBag                         = nil;
AIOI_Saved_ToggleBackpack                   = nil;
AIOI_Saved_CloseBackpack                    = nil;
AIOI_Saved_OpenBackpack                     = nil;
AIOI_Saved_OpenAllBags                      = nil;
AIOI_Saved_CloseAllBags                     = nil;
AIOI_Saved_BagSlotButton_OnClick            = nil;
AIOI_Saved_BagSlotButton_OnDrag             = nil;
AIOI_Saved_BackpackButton_OnClick           = nil;
AIOI_Saved_LootFrame_OnShow                 = nil;
-- AIOI_Saved_UseContainerItem                 = nil;



-- Loading
function AIOI_InitializeProfile() --
   if AIOI_Initialized == true then
      return
   end
   if ( UnitName('player') ) then
      PlayerName = UnitName('player').."|"..AIOI_Trim(GetCVar("realmName"));
      AIOI_LoadSettings();
      AIOI_Print(format(AIOI_MSG_INIT_s,PlayerName));
      AIOI_Initialized = true;
      AIOIFrame_UpdateLook();
      AIOI_SetFreeze();
   end
end

function AIOI_LoadSettings() --
   if ( AIOIProfile[PlayerName] == nil ) then
      AIOIProfile[PlayerName] = {};
      AIOI_Print(format(AIOI_MSG_CREATE_s,PlayerName));
   end
   AIOIColumns        = AIOI_SavedOrDefault("Columns");
   AIOIReplaceBags    = AIOI_SavedOrDefault("ReplaceBags");
   AIOIFreeze         = AIOI_SavedOrDefault("Freeze");
   AIOILock           = AIOI_SavedOrDefault("Lock");
   AIOIHighlightItems = AIOI_SavedOrDefault("HighlightItems");
   AIOIHighlightBags  = AIOI_SavedOrDefault("HighlightBags");
   AIOIBagView        = AIOI_SavedOrDefault("BagView");
   AIOIGraphics       = AIOI_SavedOrDefault("Graphics");
   AIOIBackground     = AIOI_SavedOrDefault("Background");
   AIOICount          = AIOI_SavedOrDefault("Count");
   AIOIShowTitle      = AIOI_SavedOrDefault("ShowTitle");
   AIOICash           = AIOI_SavedOrDefault("Cash");
   AIOIButtons        = AIOI_SavedOrDefault("Buttons");
   AIOIScale          = AIOI_SavedOrDefault("Scale");
   AIOIBags           = AIOI_SavedOrDefault("Bags");
   AIOIReverseStyle   = AIOI_SavedOrDefault("ReverseStyle");

   AIOI_SetScale(tonumber(AIOIScale));
end

function AIOI_SavedOrDefault(varname) --
   if PlayerName == nil or varname == nil then
      AIOI_DEBUG("ERR: nil value");
      return nil;
   end
   if AIOIProfile[PlayerName][varname] == nil then -- Setting not set
      AIOIProfile[PlayerName][varname] = getglobal("AIOI"..varname); -- Load Default
   end
   return AIOIProfile[PlayerName][varname];  -- Return Setting.
end


function AIOI_Toggle_Option(option, value, quiet, updateLook) --
   if value == nil then
      if getglobal("AIOI"..option) == 1 then
         value = 0;
      else
         value = 1;
      end
   end
   setglobal("AIOI"..option, value);
   AIOIProfile[PlayerName][option] = value;
   if not quiet or  quiet ~= 1 then
      local chat_message;
      local globalName = "AIOI_CHAT_"..string.upper(option);
      if value == 0 then
         globalName = globalName.."OFF";
      else
         globalName = globalName.."ON";
      end
      chat_message = getglobal(globalName);
      if ( chat_message ) then
         AIOI_Print(AIOI_CHAT_PREFIX..chat_message);
      else
         AIOI_DEBUG("ERROR: No global "..globalName);
      end
   end
   if AIOIGraphics == 1 then
      AIOI_OptionsFrame_CheckButton_Background:Hide();
   else
      AIOI_OptionsFrame_CheckButton_Background:Show();
   end
   if AIOIBagView == 1 and AIOIColumns < 5 then
      AIOIFrame_SetColumns(5);
   end
   if updateLook and updateLook == 1 then
      AIOIFrame_UpdateLook();
   end
   if option=="Freeze" then
      AIOI_SetFreeze()
   end
end

function AIOI_SetFreeze()
   if AIOIFreeze == 0 then
      tinsert(UISpecialFrames, "AIOIFrame"); -- Esc Closes Inventory
   else
      for key, value in pairs(UISpecialFrames) do
         if value == "AIOIFrame" then
            table.remove(UISpecialFrames, key);
         end
      end
   end
end

-- Slash commands
function AIOI_ChatCommandHandler(msg) --

   if ( ( not msg ) or ( strlen(msg) <= 0 ) ) then
      AIOIOptionsFrame:Show()
      AIOI_Print(AIOI_CHAT_COMMAND_USAGE);
      return;
   end

   local commandName, params = AIOI_Extract_NextParameter(msg);

   if ( ( commandName ) and ( strlen(commandName) > 0 ) ) then
      commandName = string.lower(commandName);
   else
      commandName = "";
   end

   if ( (strfind(commandName, "show")) or (strfind(commandName, "toggle")) ) then
      ToggleAIOIFrame();
   elseif ( strfind(commandName, "config") or strfind(commandName, "option") ) then
      AIOIOptionsFrame:Show();
   elseif ( strfind(commandName, "replace") ) then
      AIOI_Toggle_Option("ReplaceBags");
   elseif ( strfind(commandName, "freeze") ) then
      AIOI_Toggle_Option("Freeze");
   elseif ( strfind(commandName, "reverse") ) then
      AIOI_Toggle_Option("ReverseStyle", nil, nil, 1);
   elseif ( strfind(commandName, "lock") ) then
      AIOI_Toggle_Option("Lock",nil,nil,1);
   elseif ( strfind(commandName, "cash") or strfind(commandName, "money")) then
      AIOI_Toggle_Option("Cash",nil,nil,1);
   elseif ( strfind(commandName, "button") ) then
      AIOI_Toggle_Option("Buttons",nil,nil,1);
   elseif ( strfind(commandName, "title") ) then
      AIOI_Toggle_Option("ShowTitle",nil,nil,1);
   elseif ( strfind(commandName, "count") or strfind(commandName, "slot")) then
      AIOI_Toggle_Option("Count",nil,nil,1);
   elseif ( strfind(commandName, "graphic") or strfind(commandName, "art") ) then
      AIOI_Toggle_Option("Graphics",nil,nil,1);
   elseif ( strfind(commandName, "back") ) then
      AIOI_Toggle_Option("Background",nil,nil,1);
   elseif ( strfind(commandName, "bagview") ) then
      AIOI_Toggle_Option("BagView",nil,nil,1);
   elseif ( strfind(commandName, "col") ) then
      cols, params = AIOI_Extract_NextParameter(params);
      if cols then
         AIOIFrame_SetColumns(cols);
      end
   elseif ( strfind(commandName, "scale") ) then
      scale, params = AIOI_Extract_NextParameter(params);
      if tonumber(scale) and tonumber(scale) >= 0.5 and tonumber(scale) <= 1.5 then
         AIOI_SetScale(scale);
      else
         AIOI_Print("AIOI: Scale is at "..AIOIScale);
      end
   elseif ( strfind(commandName, "resetpos") ) then
      AIOIAnchorFrame:ClearAllPoints();
      AIOIAnchorFrame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMRIGHT", 0, 50);
      AIOIFrame:ClearAllPoints();
      AIOIFrame:SetPoint("BOTTOMRIGHT", "AIOIAnchorFrame", "BOTTOMRIGHT");
   else
      AIOI_Print(AIOI_CHAT_COMMAND_USAGE);
      return;
   end
end

function AIOI_SetScale(scale) --
   local pScale = tonumber(AIOIFrame:GetParent():GetScale());
   if ( pScale == nil ) then
      pScale = tonumber(GetCVar("uiscale"));
      if not pScale then
         pScale = 1;
         AIOI_DEBUG("Scale Error")
      end
   end

   AIOIFrame:SetScale(pScale * tonumber(scale));
   AIOI_Toggle_Option("Scale", scale, 1);
end

function AIOIFrame_SetColumns(col) --
   if ( type(col) ~= "number" ) then
      col = tonumber(col);
   end
   if (type(col) ~= "number" or col == 0) then
      return
   end
   if AIOIBagView == 1 and col < 5 then
      col = 5;
   end
   if ( ( col >= AIOI_COLUMNS_MIN ) and ( col <= AIOI_COLUMNS_MAX ) ) then
      AIOIColumns = col;
      AIOIProfile[PlayerName].Columns = AIOIColumns;
      AIOIFrame_UpdateLook();
   end
end

-- Get Bag Slots
function AIOI_GetBagsTotalSlots() --
   local slots = 0;
   for bag = 0, 4 do
      slots = slots + GetContainerNumSlots(bag);
   end
   return slots;
end
function AIOI_GetBagsReplacingSlots() --
   local slots = 0;
   for bag = 0, 4 do
      if AIOI_ShouldOverrideBag(bag) then
         slots = slots + GetContainerNumSlots(bag);
      end
   end
   return slots;
end
function AIOI_GetIdAsBagSlot(id) --
   -- depreciated
   local button = getglobal("AIOIFrameItem"..id);
   if button.bagIndex then
      return button.bagIndex, button.itemIndex;
   end
   return -1,-1;
end


-- Loading
function AIOIFrame_OnLoad() --
   this:RegisterEvent("MERCHANT_SHOW");
   this:RegisterEvent("MERCHANT_CLOSED");
   this:RegisterEvent("BANKFRAME_OPENED");
   this:RegisterEvent("BANKFRAME_CLOSED");
   this:RegisterEvent("AUCTION_HOUSE_SHOW");
   this:RegisterEvent("AUCTION_HOUSE_CLOSED");
   this:RegisterEvent("MAIL_SHOW");
   this:RegisterEvent("MAIL_CLOSED");
   this:RegisterEvent("TRADE_SHOW");
   this:RegisterEvent("TRADE_CLOSED");

   AIOI_Register();
   AIOI_Print(AIOI_MSG_LOADED);
   local pScale = AIOIFrame:GetParent():GetScale();
   if ( not pScale ) then pScale = 1; end
   if AIOIFrame:GetScale() ~= pScale * tonumber(AIOIScale) then
      AIOIFrame:SetScale(AIOIScale);
   end
end

function AIOI_Register() --
   SlashCmdList["AIOISLASHMAIN"] = AIOI_ChatCommandHandler;
   SLASH_AIOISLASHMAIN1 = "/allinoneinventory";
   SLASH_AIOISLASHMAIN2 = "/aioi";
   this:RegisterEvent("UNIT_NAME_UPDATE");
   this:RegisterEvent("VARIABLES_LOADED");
end

function AIOI_MyAddonsRegister() --
   if (myAddOnsFrame) then
      myAddOnsList.AIOI = {
         name = AIOI_MYADDON_NAME,
         description = AIOI_MYADDON_DESCRIPTION,
         version = AIOI_MYADDON_VERSION,
         category = MYADDONS_CATEGORY_INVENTORY,
         frame = "AIOIFrame",
         optionsframe = "AIOIOptionsFrame"
      };
   end
end

-- Hooks functions.
--  Hooking functions mean that you replace them with your own functions and then call the
--  original function at your leisure.
function AIOI_Setup_Hooks(toggle) --
   if (toggle == 1) then
      if ( ( ToggleBag ~= AIOI_Saved_ToggleBag ) and (AIOI_Saved_ToggleBag == nil) ) then
         AIOI_Saved_ToggleBag = ToggleBag;
         ToggleBag = AIOI_ToggleBag;
      end
      if ( ( OpenBag ~= AIOI_Saved_OpenBag ) and (AIOI_Saved_OpenBag == nil) ) then
         AIOI_Saved_CloseBag = CloseBag;
         CloseBag = AIOI_CloseBag;
      end
      if ( ( OpenBag ~= AIOI_Saved_OpenBag ) and ( AIOI_Saved_OpenBag == nil) ) then
         AIOI_Saved_OpenBag = OpenBag;
         OpenBag = AIOI_OpenBag;
      end
      if ( ( OpenAllBags ~= AIOI_Saved_OpenAllBags ) and (AIOI_Saved_OpenAllBags == nil) ) then
         AIOI_Saved_OpenAllBags = OpenAllBags;
         OpenAllBags = AIOI_OpenAllBags;
      end
      if ( ( CloseAllBags ~= AIOI_Saved_CloseAllBags ) and (AIOI_Saved_CloseAllBags == nil) ) then
         AIOI_Saved_CloseAllBags = CloseAllBags;
         CloseAllBags = AIOI_CloseAllBags;
      end
      if ( ( ToggleBackpack ~= AIOI_Saved_ToggleBackpack ) and ( AIOI_Saved_ToggleBackpack == nil) ) then
         AIOI_Saved_ToggleBackpack = ToggleBackpack;
         ToggleBackpack = AIOI_ToggleBackpack;
      end
      if ( ( CloseBackpack ~= AIOI_Saved_CloseBackpack ) and (AIOI_Saved_CloseBackpack == nil) ) then
         AIOI_Saved_CloseBackpack = CloseBackpack;
         CloseBackpack = AIOI_CloseBackpack;
      end
      if ( (OpenBackpack ~= AIOI_Saved_OpenBackpack ) and (AIOI_Saved_OpenBackpack == nil) ) then
         AIOI_Saved_OpenBackpack = OpenBackpack;
         OpenBackpack = AIOI_OpenBackpack;
      end
      if ( (BagSlotButton_OnClick  ~= AIOI_Saved_BagSlotButton_OnClick ) and (AIOI_Saved_BagSlotButton_OnClick == nil) ) then
         AIOI_Saved_BagSlotButton_OnClick = BagSlotButton_OnClick;
         BagSlotButton_OnClick = AIOI_BagSlotButton_OnClick;
      end
      if ( (BagSlotButton_OnDrag  ~= AIOI_Saved_BagSlotButton_OnDrag ) and (AIOI_Saved_BagSlotButton_OnDrag == nil) ) then
         AIOI_Saved_BagSlotButton_OnDrag = BagSlotButton_OnDrag;
         BagSlotButton_OnDrag = AIOI_BagSlotButton_OnDrag;
      end
      if ( (BackpackButton_OnClick  ~=  AIOI_Saved_BackpackButton_OnClick) and (AIOI_Saved_BackpackButton_OnClick == nil) ) then
         AIOI_Saved_BackpackButton_OnClick = BackpackButton_OnClick;
         BackpackButton_OnClick = AIOI_BackpackButton_OnClick;
      end
--		if ( (LootFrame_OnShow  ~= AIOI_Saved_LootFrame_OnShow ) and (AIOI_Saved_LootFrame_OnShow == nil) ) then
--			AIOI_Saved_LootFrame_OnShow = LootFrame_OnShow;
--			LootFrame_OnShow = AIOI_LootFrame_OnShow;
--		end
--		if ( (UseContainerItem ~= AIOI_Saved_UseContainerItem ) and (AIOI_Saved_UseContainerItem == nil) ) then
--			AIOI_DEBUG("hooking UseContainerItem");
--			AIOI_Saved_UseContainerItem = UseContainerItem;
--			UseContainerItem = AIOI_UseContainerItem;
--		end
--		if ( (  ~=  ) and ( == nil) ) then
--		end
--		if ( (  ~=  ) and ( == nil) ) then
--		end
   else
   end
end

-- Does things with the hooked function --
function AIOI_LootFrame_OnShow()
   AIOI_Saved_LootFrame_OnShow();
   LootFrame:Raise();
end

function AIOI_BagSlotButton_OnClick()
   AIOI_Saved_BagSlotButton_OnClick()
   AIOI_UpdateBagState();
end

function AIOI_BagSlotButton_OnDrag()
   AIOI_Saved_BagSlotButton_OnDrag()
   AIOI_UpdateBagState();
end

function AIOI_BackpackButton_OnClick()
   AIOI_Saved_BackpackButton_OnClick()
   AIOI_UpdateBagState();
end

-- Toggling Bags/Backpack
function AIOI_ShouldOverrideBag(bag)
   if (  (bag >= 0) and (bag <= 4) ) then
      if strsub(AIOIBags,(1+bag),(1+bag)) == "1" then
         return true;
      else
         return false
      end
   else
      return false;
   end
end

function AIOI_ToggleBackpack()
   AIOI_DEBUG("ToggleBackpack called");
   if ((AIOIReplaceBags == 1) and AIOI_ShouldOverrideBag(0)) then
      ToggleAIOIFrame();
      return;
   end
   AIOI_Saved_ToggleBackpack()
   AIOI_UpdateBagState();
end

function AIOI_ToggleBag(bag)
   AIOI_DEBUG("ToggleBag"..bag.." called");
   if ((AIOIReplaceBags == 1) and AIOI_ShouldOverrideBag(bag)) then
      ToggleAIOIFrame();
      return;
   else
      AIOI_Saved_ToggleBag(bag);
      AIOI_UpdateBagState();
   end
end

function AIOI_OpenBackpack()
   AIOI_DEBUG("Open Backpack called");
   if ( AIOIReplaceBags == 1 ) then
      OpenAIOIFrame();
      return;
   end
   AIOI_Saved_OpenBackpack()
   AIOI_UpdateBagState();
end

function AIOI_CloseBackpack()
   AIOI_DEBUG("Close Backpack called");
   if ( AIOIReplaceBags == 1 ) then
      if ( AIOIFreeze == 0) then
         CloseAIOIFrame();
      end
      return;
   end
   AIOI_DEBUG("Default close");
   AIOI_Saved_CloseBackpack()
   AIOI_UpdateBagState();
end

function AIOI_CloseBag(bag)
   AIOI_DEBUG("Close Bag"..bag.." called");
   if ((AIOIReplaceBags == 1) and AIOI_ShouldOverrideBag(bag)) then
   else
      AIOI_Saved_CloseBag(bag);
      AIOI_UpdateBagState();
   end
end

function AIOI_OpenBag(bag)
   if ((AIOIReplaceBags == 1) and AIOI_ShouldOverrideBag(bag)) then
   else
      AIOI_Saved_OpenBag(bag)
      AIOI_UpdateBagState();
   end
end

function AIOI_OpenAllBags(arg)
   AIOI_DEBUG("Open All Bags Called");
   if ((AIOIReplaceBags == 1)) then
      AIOI_DEBUG("Replacing function");
      ToggleAIOIFrame()
   else
      AIOI_Saved_OpenAllBags(arg);
   end
end

function AIOI_CloseAllBags(arg)
   AIOI_DEBUG("Close All Bags Called");
   if (AIOIReplaceBags == 1) then
      AIOI_DEBUG("Replacing function");
      if AIOI_Freeze == 1 then
         return
      end
      if AIOIFrame:IsVisible() then
         ToggleAIOIFrame()
      end
   else
      AIOI_Saved_CloseAllBags(arg);
   end
end

function AIOI_UseContainerItem(bag, slot)
   AIOI_DEBUG("UseContainerItem called");
   if ( not slot ) then
      local b,s = AIOI_GetIdAsBagSlot(bag);
      if ( ( b > -1 ) and ( s > -1 ) ) then
         return AIOI_Saved_UseContainerItem(b, s);
      end
   end
   return AIOI_Saved_UseContainerItem(bag, slot);
end

-- Toggle Frame
function ToggleAIOIFrame() --
   if ( AIOIFrame:IsVisible() ) then
      CloseAIOIFrame();
   else
      OpenAIOIFrame();
   end
   AIOI_UpdateBagState();
end

function CloseAIOIFrame() --
   if ( AIOIFrame:IsVisible() ) then
      AIOIFrame:Hide();
   end
   AIOI_UpdateBagState();
end

function OpenAIOIFrame() --
   AIOIFrame_UpdateLookIfNeeded();
   AIOIFrame:Show();
   AIOI_UpdateBagState();
end


-- Update Bag State: Setting Checks
function AIOI_IsBagOpen(id)
   local formatStr = "ContainerFrame%d";
   local frame = nil;
   for i = 1, NUM_CONTAINER_FRAMES do
      frame = getglobal(format(formatStr, i));
      if ( ( frame ) and ( frame:IsVisible() ) and ( frame:GetID() == id ) ) then
         return true;
      end
   end
   return false;
end

function AIOI_GetBagState(toggle)
   if ( ( toggle == true ) or ( toggle == 1 ) ) then
      return 1;
   else
      return 0;
   end
end

function AIOI_UpdateBagState()
   local visible = AIOIFrame:IsVisible();
   local val = strsub(AIOIBags, 1,1);
   MainMenuBarBackpackButton:SetChecked(AIOI_GetBagState((val=="1" and visible) or AIOI_IsBagOpen(0)));
   AIOIBackpackButton:SetChecked(AIOI_GetBagState((val=="1" and visible)));
   local bagButton = nil;
   local mibagButton = nil
   local formatStr = "CharacterBag%dSlot";
   local formatStr2= "AIOI_____Bag%dSlot";
   for i = 0, 3 do
      local val = strsub(AIOIBags, i+2,i+2);
      bagButton   = getglobal(format(formatStr, i));
      mibagButton = getglobal(format(formatStr2, i));
      if ( bagButton ) then
         bagButton:SetChecked(AIOI_GetBagState((val=="1" and visible) or AIOI_IsBagOpen(i+1)));
         mibagButton:SetChecked(AIOI_GetBagState((val=="1" and visible) ));
      end
   end
end


-- Highlighting
function AIOI_HighlightBagItems(bagID, off)
   if AIOIHighlightItems == 0 then
      return
   end
   local found;
   local itemButton;
   for i = 1,AIOI_MAX_ID do
      local bag, slot;
      itemButton = getglobal("AIOIFrameItem"..i);
      bag = itemButton.bagIndex;
      if bag == bagID then
         found = true;
         if off and off == 1 then
            itemButton:UnlockHighlight();
         else
            itemButton:LockHighlight();
         end
      else
         if found then
            break;
         end
      end
   end
end

function AIOI_HighlightItemBag(bagID, off)
   if AIOIHighlightBags == 0 then
      return
   end
   local BagButton;
   if bagID == 0 then
      BagButton= getglobal("AIOIBackpackButton");
   else
      BagButton= getglobal("AIOI_____Bag"..(bagID-1).."Slot");
   end
   -- note("YO", BagButton:GetName());
   if off then
      BagButton:UnlockHighlight();
   else
      BagButton:LockHighlight();
   end
end


-- Update Visuals, toggled options
function AIOITitle_Update()
   if AIOICash == 1 then
      AIOIFrameMoneyFrame:Show();
   else
      AIOIFrameMoneyFrame:Hide();
   end
   AIOISlots_Update();

   if AIOIShowTitle == 0 then
      AIOIFrameName:Hide();
   else
      AIOIFrameName:Show();
      if ( UnitName('player') ) then
         local player, realm = UnitName('player'),AIOI_Trim(GetCVar("realmName"));
         if (AIOIColumns < 5 ) or (AIOIGraphics == 1 and AIOIColumns < 7) then
            AIOIFrameName:SetText(AIOI_TITLE);
         else
            AIOIFrameName:SetText(format(AIOI_TITLE_S, player));
         end
         AIOIFrameName:SetTextColor(1.0, 1.0, 1.0);
      end
   end
end

function AIOISlots_Update()
   local i, j, totalSlots= 0, 0, 0;
   local takenSlots = 0;

   for i = 0, 4 do
      local slots = GetContainerNumSlots(i)
      if AIOI_ShouldOverrideBag(i) then
         totalSlots = totalSlots + slots;
         for j = 1, slots do
            if GetContainerItemInfo(i,j) then
               takenSlots=takenSlots + 1;
            end
         end
      end
   end
   if AIOICount == 0 then
      AIOIFrameSlots:Show();
      AIOIFrameSlots:SetText(format(AIOI_SLOTS_DD, (totalSlots-takenSlots),totalSlots));
   elseif AIOICount == 1 then
      AIOIFrameSlots:Show();
      AIOIFrameSlots:SetText(format(AIOI_SLOTS_DD, (takenSlots),totalSlots));
   else
      AIOIFrameSlots:Hide();
   end
end

function AIOI_SetLock()
   if AIOILock == 1 then
      AIOILockNormalTexture:SetTexture("Interface\\AddOns\\AllInOneInventory\\Skin\\LockButton-Locked-Up");
   else
      AIOILockNormalTexture:SetTexture("Interface\\AddOns\\AllInOneInventory\\Skin\\LockButton-Unlocked-Up");
   end
   AIOIFrame:EnableMouse(1-AIOILock);
end

function AIOI_ShowButtons()
   if AIOIButtons == 1 then
      AIOIFrameCloseButton:Show();
      AIOIFrameLockButton:Show();
      AIOIFrameHideBagsButton:Show();
   else
      AIOIFrameCloseButton:Hide();
      AIOIFrameLockButton:Hide();
      AIOIFrameHideBagsButton:Hide();
   end
   -- always show the close button for blizz style graphics.
   if AIOIGraphics == 1 then
      AIOIFrameCloseButton:Show();
   end
end

function AIOI_SetGraphics()
   if AIOIGraphics == 1 then
      AIOIFrame:SetBackdropColor(0,0,0,0);
      AIOIFrame:SetBackdropBorderColor(0,0,0,0);

      AIOIFrameTextureTopLeft:Show();
      AIOIFrameTextureTopCenter:Show();
      AIOIFrameTextureTopRight:Show();
      AIOIFrameTextureLeft:Show();
      AIOIFrameTextureCenter:Show();
      AIOIFrameTextureRight:Show();
      AIOIFrameTextureBottomLeft:Show();
      AIOIFrameTextureBottomCenter:Show();
      AIOIFrameTextureBottomRight:Show();

      AIOIFrameMoneyFrame:ClearAllPoints();
      AIOIFrameMoneyFrame:SetPoint("BOTTOMRIGHT", "AIOIFrame", "BOTTOMRIGHT", -5, 4);
      AIOIFrameSlots:ClearAllPoints();
      AIOIFrameSlots:SetPoint("BOTTOMLEFT", 7, 5);

      AIOIFrameName:ClearAllPoints();
      AIOIFrameName:SetPoint("TOP", "AIOIFrame", "TOP", 0, -4);
      AIOIFrameCloseButton:ClearAllPoints();
      AIOIFrameCloseButton:SetPoint("TOPRIGHT", "AIOIFrame", "TOPRIGHT", 9, 4);
   else
      if AIOIBackground==1 then
         AIOIFrame:SetBackdropColor(0,0,0,1.0);
         AIOIFrame:SetBackdropBorderColor(1,1,1,1.0);
      else
         AIOIFrame:SetBackdropColor(0,0,0,0);
         AIOIFrame:SetBackdropBorderColor(1,1,1,0);
      end

      AIOIFrameTextureTopLeft:Hide();
      AIOIFrameTextureTopCenter:Hide();
      AIOIFrameTextureTopRight:Hide();
      AIOIFrameTextureLeft:Hide();
      AIOIFrameTextureCenter:Hide();
      AIOIFrameTextureRight:Hide();
      AIOIFrameTextureBottomLeft:Hide();
      AIOIFrameTextureBottomCenter:Hide();
      AIOIFrameTextureBottomRight:Hide();

      AIOIFrameMoneyFrame:ClearAllPoints();
      AIOIFrameMoneyFrame:SetPoint("BOTTOMRIGHT", "AIOIFrame", "BOTTOMRIGHT", -5, 8);
      AIOIFrameSlots:ClearAllPoints();
      AIOIFrameSlots:SetPoint("BOTTOMRIGHT", "AIOIFrame", "BOTTOMRIGHT", -20, 24);

      AIOIFrameName:ClearAllPoints();
      AIOIFrameName:SetPoint("TOPLEFT", "AIOIFrame", "TOPLEFT", 5, -6);
      AIOIFrameCloseButton:ClearAllPoints();
      AIOIFrameCloseButton:SetPoint("TOPRIGHT", "AIOIFrame", "TOPRIGHT", 2, 2);
   end
end


-- Get Height and Width
function AIOIFrame_GetAppropriateHeight(rows, columns)
   -- Border 6 on top, 6 on bottom
   -- First row is 28 pixels down.
   -- BASE_HEIGHT = 44
   if AIOIBagView == 1 then
      rows = rows+1;
   end

   -- this fixes the condition where the last item would be outside the frame.
   if (mod(AIOIFrame.size, columns) == 0) then
      rows = rows+1;
   end
   -- this fixes the condition where the last item overlaps the money info.
   -- note(mod(AIOIFrame.size, columns)-1) -- how many items are on the last row...
   if (AIOIGraphics == 0 and (AIOICash == 1 or AIOICount == 0 or AIOICount == 1))
      and (mod(AIOIFrame.size, columns)-1 > math.max(0, columns-4)) then
      rows = rows+1;
   end

   local height = 12;
   if AIOIButtons==1 or AIOIShowTitle==1 or AIOIGraphics == 1 then
      height = height + AIOI_TOP_HEIGHT;
   end
   if AIOIGraphics == 1 and ((AIOICash==1) or (AIOICount > -1))  then
      height = height + AIOI_BOTTOM_HEIGHT;
   end
   height = height + (-AIOI_ITEM_OFFSET_Y) * rows;
  return height;
end

function AIOIFrame_GetAppropriateWidth(cols)
   return 12 + ( AIOI_ITEM_OFFSET_X * cols );
end


-- Update Look
function AIOIFrame_UpdateLookIfNeeded()
   local slots = AIOI_GetBagsTotalSlots();
   if ( ( not AIOIFrame.size ) or ( slots ~= AIOIFrame.size ) ) then
      AIOIFrame_UpdateLook();
   end
end

function AIOIFrame_UpdateLook()
   --local frameSize = AIOI_GetBagsTotalSlots();
   local frameSize = AIOI_GetBagsReplacingSlots();
   AIOITitle_Update();  -- Update Title
   AIOI_SetLock();      -- Set Lock Icon
   AIOI_ShowButtons();  -- Show or Hide Buttons

   AIOIFrame.size = frameSize;

   local name = AIOIFrame:GetName();
   -- Update Size Anchor to bottom right
   local columns, rows = AIOIColumns, ceil(frameSize / AIOIColumns);

   local height, width = AIOIFrame_GetAppropriateHeight(rows, columns), AIOIFrame_GetAppropriateWidth(columns);

   AIOIFrame:SetHeight(height);
   AIOIFrame:SetWidth(width);

   AIOI_SetGraphics(); -- Turn on or Off Blizzard Graphics

   -- Border
   local First_Y = -6 ;
   -- Title
   if AIOIShowTitle==1 or AIOIButtons == 1 or AIOIGraphics == 1 then
      First_Y = First_Y - AIOI_TOP_HEIGHT;
   end

   if AIOIGraphics == 1 then
      First_Y = First_Y - AIOI_ITEM_OFFSET_Y
   end

   -- BagButtons
   AIOIBagButtonsBar:ClearAllPoints();
   if AIOIGraphics == 1 then
      First_Y = First_Y + AIOI_ITEM_OFFSET_Y;
      if AIOIColumns == 5 then
         AIOIBagButtonsBar:SetPoint("TOP", "AIOIFrame", "TOP", 0, First_Y);
      else
         AIOIBagButtonsBar:SetPoint("TOP", "AIOIFrame", "TOP", 20, First_Y);
      end
   else
      AIOIBagButtonsBar:SetPoint("TOP", "AIOIFrame", "TOP", 0, First_Y);
   end

   if AIOIBagView == 1 then
      First_Y = First_Y + AIOI_ITEM_OFFSET_Y;
      AIOIBagButtonsBar:Show();
   else
      AIOIBagButtonsBar:Hide();
   end

   local curCol;
   if AIOIReverseStyle==1 then
      curCol=columns-mod(frameSize, columns)+1;
   else
      curCol=1;
   end
   local curRow=1;
   for j=1, frameSize, 1 do
      local itemButton = getglobal(name.."Item"..j);
      -- itemButton:SetID(j);
      -- Set first button
      itemButton:ClearAllPoints();
      local dx, dy, baseframe;
      -- first item!
      if ( j == 1 ) then
         baseframe = name;
         if AIOIReverseStyle == 1 then
            dx = (curCol-1)*AIOI_ITEM_OFFSET_X + AIOI_FIRST_ITEM_OFFSET_X;
            dy = First_Y;
         else
            if AIOIGraphics == 1 and AIOIBagView == 0 then
               curCol = curCol + 1
               dx = AIOI_FIRST_ITEM_OFFSET_X + AIOI_ITEM_OFFSET_X;
            else
               dx = AIOI_FIRST_ITEM_OFFSET_X;
            end
            dy = First_Y;
         end
      -- everything else
      else
         -- have it start on a new row
         if ( mod((curCol+j-2), columns) == 0 ) then
            baseframe = "AIOIFrame";
            dx = AIOI_FIRST_ITEM_OFFSET_X;
            dy = First_Y + AIOI_ITEM_OFFSET_Y*curRow;
            curRow=curRow+1;

         -- starts based on previous item
         else
            baseframe = name.."Item"..(j - 1);
            dy = 0;
            dx = AIOI_ITEM_OFFSET_X;
         end
      end
      itemButton:SetPoint("TOPLEFT", baseframe, "TOPLEFT", dx, dy);

      itemButton:Show();
   end
   local button = nil;
   for i = frameSize+1, AIOI_MAX_ID do
      button = getglobal("AIOIFrameItem"..i);
      if ( button ) then
         button:Hide();
      end
   end
end

function AIOIFrame_Update(frame)
   local frame = getglobal("AIOIFrame");
   AIOIFrame_UpdateLookIfNeeded();
   local name = frame:GetName();
   local index = 0;
   AIOISlots_Update();
   local altbag=0;
   for bag=0, 4 do
      if AIOI_ShouldOverrideBag(bag) and GetContainerNumSlots(bag)>0 then
         altbag=math.abs(altbag-1);
         local slotid = 1;
         for slot=1, GetContainerNumSlots(bag) do

            index = index + 1;
            local itemButton = getglobal(name.."Item"..index);
            local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag, slot);
            itemButton.bagIndex, itemButton.itemIndex = bag, slot; -- Save the slot and bag, MUCH easier then looking it up every time.
            local dummybag = getglobal(name.."Bag"..bag);
            itemButton:SetParent(dummybag);
            itemButton:SetID(slotid);
            slotid = slotid + 1;
            SetItemButtonTexture(itemButton, texture);
            SetItemButtonCount(itemButton, itemCount);
            SetItemButtonDesaturated(itemButton, locked, 1, 1, 1);

            AIOI_UpdateBorder(itemButton, quality, bag)

            if ( texture ) then
               local cooldown = getglobal(itemButton:GetName().."Cooldown");
               local start, duration, enable = GetContainerItemCooldown(bag, slot);
               CooldownFrame_SetTimer(cooldown, start, duration, enable);
               if ( duration > 0 and enable == 0 ) then
                  SetItemButtonTextureVertexColor(itemButton, 0.4, 0.4, 0.4);
               end
            else
               getglobal(name.."Item"..index.."Cooldown"):Hide();
            end

            itemButton.readable = readable;
            local showSell = nil;
            if ( GameTooltip:IsOwned(itemButton) ) then
               if ( texture ) then
                  local hasCooldown, repairCost = GameTooltip:SetBagItem(bag, slot);
                  if ( hasCooldown ) then
                     itemButton.updateTooltip = TOOLTIP_UPDATE_TIME;
                  else
                     itemButton.updateTooltip = nil;
                  end
                  if ( InRepairMode() and (repairCost > 0) ) then
                     GameTooltip:AddLine(TEXT(REPAIR_COST), "", 1, 1, 1);
                     SetTooltipMoney(GameTooltip, repairCost);
                     GameTooltip:Show();
                  elseif ( MerchantFrame:IsVisible() and not locked) then
                     showSell = 1;
                  end
               else
                  GameTooltip:Hide();
               end
               if ( showSell ) then
                  ShowContainerSellCursor(bag, slot);
               elseif ( readable ) then
                  ShowInspectCursor();
               else
                  ResetCursor();
               end
            end
         end
      end
   end
end
