local TEXTURE = "i"
local LINK = "l"
local SLOTS = "sl"
local COUNT = "c"
local QUALITY = "q"
local SOULBOUND = "sb"
local MADEBY = "m"
local COOLDOWN = "d"
local CDDURATION = "cd"
local CDSTART = "s"
local CDENABLED = "e"
local MONEY = "money"

function AIOB_SaveItemData(AIOBItem, itemLink, texture, Slots, Count, Quality, soulbound, madeBy, Cooldown)
   -- example:   "|cff1eff00|Hitem:16202:0:0:0:0:0:0:0|h[Lesser Eternal Essence]|h|r"

   -- note("SAVING:", itemLink, texture, Slots, Count, Quality, soulbound, madeBy, Cooldown)
   AIOBItem[LINK] = itemLink;
   AIOBItem[TEXTURE] = texture;
   AIOBItem[SLOTS] = Slots;
   AIOBItem[COUNT] = Count ;
   AIOBItem[QUALITY] = Quality;
   AIOBItem[SOULBOUND] = soulbound;
   AIOBItem[MADEBY] = madeBy;
   AIOBItem[COOLDOWN] = Cooldown;
end


function AIOB_GetCooldown(item)
   if item[COOLDOWN] then
      local cooldownInfo = item[COOLDOWN];
      local CoolDownRemaining;
      if cooldownInfo and cooldownInfo[CDDURATION] and cooldownInfo[CDSTART] then
         CoolDownRemaining = cooldownInfo[CDDURATION] - (GetTime() - cooldownInfo[CDSTART]);
      else
         CoolDownRemaining = 0;
      end
      if CoolDownRemaining <= 0 then
         item[CDDURATION] = nil;
      else
         return cooldownInfo;
      end
   end
   return nil;
end

function AIOI_GetCoolDownDuration(item)
   return item[CDDURATION];
end

function AIOI_GetCoolDownStart(item)
   return item[CDSTART];
end

function AIOI_GetCoolDownEnabled(item)
   return item[CDENABLED];
end

function AIOI_GetItemCount(item)
   return item[COUNT];
end

function AIOI_GetItemTexture(item)
   return item[TEXTURE];
end

function AIOI_GetItemSlots(item)
   return item[SLOTS];
end

function AIOI_GetItemMadeBy(item)
   return item[MADEBY];
end

function AIOI_GetItemSoulBound(item)
   return item[SOULBOUND];
end

function AIOB_SetCooldown(start, duration, enabled)
   return {
            [CDSTART] = start,
            [CDDURATION] = duration,
            [CDENABLED] = enabled
          };
end

function AIOB_GetItemQuality(item)
   return item[QUALITY]
end

function AIOB_SaveMoney()
   if ( AIOB_getPlayerName() ) then
      local playerName = AIOB_getPlayerName();
      if ( AIOBProfile[playerName] ) then
         AIOB_SetMoney(playerName, GetMoney() or 0)
      end
      if ( AIOB_MoneyFrame:IsVisible() ) then
         MoneyFrame_Update("AIOB_MoneyFrame", AIOB_GetMoney(AIOB_getBankPlayer()));
         AIOB_UpdateTotalMoney();
      end
   end
end

function AIOB_SetMoney(player, money)
   AIOBProfile[AIOB_getPlayerName()][MONEY] = money
end


function AIOB_GetMoney(player)
   return AIOBProfile[player[MONEY]];
end


function AIOB_GetTooltipData()
   local soulbound = nil;
   local madeBy = nil;
   local field;
   local left, right;

   for index = 1, 15, 1 do
      field = getglobal("AIOBHiddenTooltipTextLeft"..index);
      if( field and field:IsVisible() and field:GetText() ) then
         left = field:GetText();
      else
         left = "";
      end
      field = getglobal("AIOBHiddenTooltipTextRight"..index);
      if( field and field:IsVisible() ) then
         right = field:GetText();
      else
         right = "";
      end
      if ( string.find(left, ITEM_SOULBOUND) ) then
         soulbound = 1;
      end
      local iStart, iEnd, val1 = string.find(left, "<Made by (.+)>");
      if (val1) then
         madeBy = val1;
      end
   end
   return soulbound, madeBy;
end

-- Get Link and Get Hyperlink - maintain independance.
function AIOB_GetLink(item)
   if item and item[LINK] then
      return item[LINK]
   end
   return nil;
end

function AIOB_GetHyperlink(item)
   if item and item[LINK] then
      -- note(string.gsub(item[LINK], "|", "  "));
      local _, _, myLink= string.find(item[LINK], "(item:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+)|h");
      return myLink
   end
   return nil
end

function AIOI_GetItemName(item)
   if item and item[LINK] then
       -- note(string.gsub(item[LINK], "|", "  "));
      local _, _, name= string.find(item[LINK], "|h%[(.+)%]|h|r");
      return name
   end
   return nil
end



function AIOBFrame_SaveBagInfo(currPlayer, bagIndex, bagName)
   if bagName=="Bank" then
      return 28;
   end
   local bagNum_Slots = GetContainerNumSlots(bagIndex);
   local bagNum_ID    = BankButtonIDToInvSlotID(bagIndex, 1);
   local itemLink     = GetInventoryItemLink("player", bagNum_ID);
   local texture      = GetInventoryItemTexture("player", bagNum_ID);
   local hasCooldown, repairCost = AIOBHiddenTooltip:SetInventoryItem("player", bagNum_ID);
   local soulbound, madeBy = AIOB_GetTooltipData();
   if (itemLink) then
      currPlayer[bagName]= {};
      AIOB_SaveItemData(currPlayer[bagName], itemLink, strsub(texture,17), bagNum_Slots, _ , _ , soulbound, madeBy, _);
      return bagNum_Slots;
   else
      currPlayer[bagName] = nil;
      return 0;
   end
end

function	AIOBSaveBagItem(currPlayer, bagNewIndex, itemIndex, bagName)
   local itemLink = GetContainerItemLink(bagNewIndex, itemIndex);
   local texture, itemCount, _, itemQuality = GetContainerItemInfo(bagNewIndex, itemIndex);
   -- note("SAVING", bagNewIndex, itemIndex, bagName, itemLink, texture)
   local hasCooldown, repairCost;
   if bagNewIndex == BANK_CONTAINER then
      local newIndex = 39 + itemIndex;
      hasCooldown, repairCost = AIOBHiddenTooltip:SetInventoryItem("player", newIndex);
   else
      hasCooldown, repairCost = AIOBHiddenTooltip:SetBagItem(bagNewIndex, itemIndex);
   end
   local start, duration, enable = GetContainerItemCooldown(bagNewIndex, itemIndex);
   local soulbound, madeBy = AIOB_GetTooltipData();
   local cooldown;

   if hasCooldown and enable > 0 then
      cooldown = AIOB_SetCooldown(start, duration, enable);
   end
   if (itemLink) then
      -- note("INFO: ", bagName, itemIndex)
      currPlayer[bagName][itemIndex] = {};
      AIOB_SaveItemData(currPlayer[bagName][itemIndex], itemLink, strsub(texture,17), _, itemCount, itemQuality, soulbound, madeBy, cooldown);
   else
      currPlayer[bagName][itemIndex] = nil;
   end
end

function AIOBFrame_SaveItems()
   local currPlayer=AIOBProfile[AIOB_getPlayerName()];
   if currPlayer == nil then
      return;
   end
   if not AIOB_GetAtBank() then
      return;
   end
   AIOI_DEBUG("SaveItems");
   currPlayer.Bags, _ = GetNumBankSlots();
   local bagName, bagMaxIndex, bagNewIndex;
   for bagNum = 0, currPlayer.Bags do
      if bagNum == 0 then -- Is it the bank?
         bagName = "Bank";
         bagNewIndex = BANK_CONTAINER;
      else	-- It's in a bag slot
         bagName = "Bag"..(4+bagNum);
         bagNewIndex = (4+bagNum);
      end
      if (not currPlayer[bagName]) then
         AIOI_DEBUG("Clearing "..bagName);
         currPlayer[bagName] = {} ;
      end
      -- note("BAGNUM ", bagNum, bagName, "INDEX ", bagNewIndex)
      bagMaxIndex = AIOBFrame_SaveBagInfo(currPlayer, bagNewIndex, bagName);
      for itemIndex = 1, bagMaxIndex do
         AIOBSaveBagItem(currPlayer, bagNewIndex, itemIndex, bagName);
      end
   end
   for bagNum = 5+currPlayer.Bags, 11 do
      currPlayer["Bag"..bagNum] = nil;
   end
   AIOB_SaveMoney();
   AIOBFrame_PopulateFrame();
end

function AIOI_DEBUG(msg)
   -- If Debug is not set, just skip it.
   if ( not AIOIDEBUG or AIOIDEBUG == 0 ) then
      return;
   end

   msg = "*** DEBUG(AIOI): "..msg;
   if ( DEFAULT_CHAT_FRAME ) then
      DEFAULT_CHAT_FRAME:AddMessage(msg, 1.0, 1.0, 0.0);
   end
end

-- Text related functions.
function AIOI_Trim (s)
   return (string.gsub(s, "^%s*(.-)%s*$", "%1"));
end

function AIOI_Print(msg,r,g,b,frame,id,unknown4th)
   --if ( Print ) then
   --	Print(msg, r, g, b, frame, id, unknown4th);
   --	return;
   --end
   if(unknown4th) then
      local temp = id;
      id = unknown4th;
      unknown4th = id;
   end

   if (not r) then r = 1.0; end
   if (not g) then g = 1.0; end
   if (not b) then b = 1.0; end
   if not frame then
      if DEFAULT_CHAT_FRAME then
         frame = DEFAULT_CHAT_FRAME
      else
         AIOI_DEBUG("quitting print");
         return
      end
   end
   if type(msg) == "table" then
      for key, value in pairs(msg) do
         frame:AddMessage(value, r, g, b,id,unknown4th);
      end
   else
      frame:AddMessage(msg, r, g, b,id,unknown4th);
   end
end


AIOI_Item_Qualities = {
	[0] = "none",
	[1] = "Poor",
	[2] = "white",
	[3] = "green",
	[4] = "blue",
	[5] = "epic",
	[6] = "legendary",
};

function AIOI_UpdateBorder(itemButton, itemQuality, bag)
   local color = {
      ["r"]=0.5,
      ["g"]=0.5,
      ["b"]=0.5 };

   if ( itemQuality ) then
      if itemQuality == -1 then
         color = {
            ["r"]=1,
            ["g"]=1,
            ["b"]=1 } -- white
      elseif itemQuality == 1 then
         color = {
            ["r"]=0.5,
            ["g"]=0.5,
            ["b"]=0.5 } -- grey
      elseif itemQuality == 2 then
         color = {
            ["r"]=0.0,
            ["g"]=1.0,
            ["b"]=0.0 } -- green
      elseif itemQuality == 3 then
         color = {
            ["r"]=0.5,
            ["g"]=0.5,
            ["b"]=1.0 } --  blue

      elseif itemQuality == 4 then
         color = {
            ["r"]=0.7,
            ["g"]=0.1,
            ["b"]=1.0 } -- purple
      else
         color = {
            ["r"]=0.0,
            ["g"]=0.0,
            ["b"]=0.0 }  -- black
      end
   end

   --modify bag colors...
   if bag and GetBagName(bag) then
      local bagName = GetBagName(bag)
      -- If it is a Quiver/AmmoBag, etc.
      if ( string.find(bagName, "Quiver") or string.find(bagName, "Ammo")
         or string.find(bagName, "Shot")  or string.find(bagName, "Bandolier")) then
         color = {
            ["r"]=1.0,
            ["g"]=1.0,
            ["b"]=0.0 }
      elseif (string.find(bagName, "Soul") or string.find(bagName, "Felcloth")) then
         color = {
            ["r"]=1.0,
            ["g"]=0.0,
            ["b"]=0.0 }
      elseif (string.find(bagName, "Herb")) then
         color = {
            ["r"]=0.0,
            ["g"]=1.0,
            ["b"]=0.0 }
      elseif (string.find(bagName, "Mine")) then
         color = {
            ["r"]=0.0,
            ["g"]=1.0,
            ["b"]=1.0 }
      elseif (string.find(bagName, "Enchanted")) then
         color = {
            ["r"]=0.0,
            ["g"]=0.0,
            ["b"]=1.0 }
      end
   end
   SetItemButtonNormalTextureVertexColor(itemButton, color.r, color.g, color.b);
end

function AIOI_Extract_NextParameter(msg) --
   local params = msg;
   local command = params;
   local index = strfind(command, " ");
   if ( index ) then
      command = strsub(command, 1, index-1);
      params = strsub(params, index+1);
   else
      params = "";
   end
   return command, params;
end


function AIOB_GetCooldownString(cooldownInfo)
   local CoolDownRemaining = AIOI_GetCoolDownDuration(cooldownInfo) - (GetTime() - AIOI_GetCoolDownStart(cooldownInfo));
   -- 60 secs in a min
   -- 3600 secs in an hour
   -- 86400 secs in a day
   local days, hours, minutes, seconds;
   days = math.floor(CoolDownRemaining / 86400);
   CoolDownRemaining = CoolDownRemaining - 86400 * days;
   hours = math.floor(CoolDownRemaining / 3600);
   CoolDownRemaining = CoolDownRemaining - 3600 * hours;
   minutes = math.floor(CoolDownRemaining / 60);
   seconds = math.floor(CoolDownRemaining - 60 * minutes);
   if days > 0 then
      return format(ITEM_COOLDOWN_TIME_DAYS_P1, days+1);
   elseif hours > 0 then
      return format(ITEM_COOLDOWN_TIME_HOURS_P1, hours+1);
   elseif minutes > 0 then
      return format(ITEM_COOLDOWN_TIME_MIN, minutes+1);
   else
      return format(ITEM_COOLDOWN_TIME_SEC, seconds);
   end
end
