-- -----------------------------------------------------------------
-- File: AIOI_Options.lua
--
-- Purpose: Functions for AIOI WoW Window.
--
-- Author: Arnvid Karstad - ievil - Asys @ Turalyon_EU
--         Most code exists in it's original form in this file from Svarten and Ramble.
--
-- Credits:
--   Sarf, for the original concept.
--   Svarten and Ramble for bringing the original MyInventory to life
--   Roger V, for the updated ContainerFrameItemButtonTemplate fix
-- -----------------------------------------------------------------

function AIOIOptionsFrame_OnLoad()
	tinsert(UISpecialFrames, "AIOIOptionsFrame"); -- Esc Closes Inventory
	UIPanelWindows["AIOIOptionsFrame"] = {area = "center", pushable = 0};

end

function AIOIOptionsFrame_OnShow()
	-- Load Checks
	AIOI_OptionsFrame_CheckButton_ReplaceBags:SetChecked(AIOIReplaceBags);
	AIOI_OptionsFrame_CheckButton_Graphics:SetChecked(AIOIGraphics);
	AIOI_OptionsFrame_CheckButton_Background:SetChecked(AIOIBackground);
	AIOI_OptionsFrame_CheckButton_HighlightBags:SetChecked(AIOIHighlightBags);
	AIOI_OptionsFrame_CheckButton_HighlightItems:SetChecked(AIOIHighlightItems);
	AIOI_OptionsFrame_CheckButton_ShowTitle:SetChecked(AIOIShowTitle);
	AIOI_OptionsFrame_CheckButton_Cash:SetChecked(AIOICash);
	AIOI_OptionsFrame_CheckButton_Buttons:SetChecked(AIOIButtons);
	AIOI_OptionsFrame_CheckButton_BagView:SetChecked(AIOIBagView);
	AIOI_OptionsFrame_CheckButton_Freeze:SetChecked(AIOIFreeze);

   if AIOICount == 1 then
		AIOI_OptionsFrame_CheckButton_CountUsed:SetChecked(1);
		AIOI_OptionsFrame_CheckButton_CountFree:SetChecked(0);
		AIOI_OptionsFrame_CheckButton_CountOff:SetChecked(0);

   elseif AIOICount == 0 then
		AIOI_OptionsFrame_CheckButton_CountUsed:SetChecked(0);
		AIOI_OptionsFrame_CheckButton_CountFree:SetChecked(1);
		AIOI_OptionsFrame_CheckButton_CountOff:SetChecked(0);

   else
		AIOI_OptionsFrame_CheckButton_CountUsed:SetChecked(0);
		AIOI_OptionsFrame_CheckButton_CountFree:SetChecked(0);
		AIOI_OptionsFrame_CheckButton_CountOff:SetChecked(1);
	end

   if AIOIGraphics == 1 then
      AIOI_OptionsFrame_CheckButton_Background:Hide();
   else
      AIOI_OptionsFrame_CheckButton_Background:Show();
   end

	AIOI_OptionsFrame_Slider_Cols:SetMinMaxValues(AIOI_COLUMNS_MIN, AIOI_COLUMNS_MAX);
	AIOI_OptionsFrame_Slider_Cols:SetValueStep(1);
	AIOI_OptionsFrame_Slider_ColsText:SetText("Columns");
	AIOI_OptionsFrame_Slider_ColsLow:SetText(AIOI_COLUMNS_MIN);
	AIOI_OptionsFrame_Slider_ColsHigh:SetText(AIOI_COLUMNS_MAX);

	AIOI_OptionsFrame_Slider_Scale:SetMinMaxValues(0.5, 1.5);
	AIOI_OptionsFrame_Slider_Scale:SetValueStep(0.01);
	AIOI_OptionsFrame_Slider_ScaleText:SetText("Scale");
	AIOI_OptionsFrame_Slider_ScaleLow:SetText("0.5");
	AIOI_OptionsFrame_Slider_ScaleHigh:SetText("1.5");
	AIOI_OptionsFrame_Slider_Cols:SetValue(AIOIColumns);
	AIOI_OptionsFrame_Slider_Scale:SetValue(AIOIScale);
end

function AIOI_OptionCheck_OnLoad()
	local option = strsub(this:GetName(), 31);
	-- Set Check button text.
	local label = getglobal(this:GetName().."Text");
	local optiontext = "AIOI_CHECKTEXT_"..string.upper(option);
	if getglobal(optiontext) then
		label:SetText(getglobal(optiontext));
	else
		label:SetText(option);
	end
	-- Set tooltip text :
	local optiontip = getglobal("AIOI_CHECKTIP_"..string.upper(option));
	if optiontip then
		this.tooltipText = optiontip;
	end
end

function AIOI_OptionCheck_OnClick()
	local value;
	if ( this:GetChecked() ) then
		PlaySound("igMainMenuOptionCheckBoxOff");
		value = 1;
	else
		PlaySound("igMainMenuOptionCheckBoxOn");
		value = 0;
	end
	--Set Option
	local option = strsub(this:GetName(), 31);
	AIOI_Toggle_Option(option, value, 1);
	AIOIFrame_UpdateLook(AIOI_GetBagsTotalSlots());
end

function AIOI_CountCheck_OnClick()
	local value;
	this:SetChecked(1);
	if AIOI_OptionsFrame_CheckButton_CountUsed ~= this then
		AIOI_OptionsFrame_CheckButton_CountUsed:SetChecked(0);
	else
		value = 1
	end
	if AIOI_OptionsFrame_CheckButton_CountFree ~= this then
		AIOI_OptionsFrame_CheckButton_CountFree:SetChecked(0);
	else
		value = 0
	end
	if AIOI_OptionsFrame_CheckButton_CountOff ~= this then
		AIOI_OptionsFrame_CheckButton_CountOff:SetChecked(0);
	else
		value = -1
	end
	AIOI_Toggle_Option("Count", value, 1);
	AIOIFrame_UpdateLook(AIOI_GetBagsTotalSlots());
end

function AIOI_OptionsFrame_Slider_Cols_OnValueChanged()
	if this:GetValue() ~= AIOIColumns then
		AIOIFrame_SetColumns(this:GetValue());
	end
end

function AIOI_OptionsFrame_Slider_Scale_OnValueChanged()
	if this:GetValue() ~= AIOIScale then
		AIOI_SetScale(this:GetValue());
	end
end

function AIOI_Check_OnEnter()
	if ( this.tooltipText ) then
		AIOI_OptionsFrame_Help_Text:SetText(this.tooltipText);
		AIOI_OptionsFrame_Help_Text:SetWidth(350);
	end
end
function AIOI_Check_OnLeave()
	AIOI_OptionsFrame_Help_Text:SetText("");
end
